import java.io.File;
import java.util.*;

/**
 * @author Roman Elizarov
 */
public class SolMap {
	private static final String SEP = File.separator;
	private static final String PROBLEMS_DIR = "../problems/".replace("/", SEP);
	private static final String TESTS_DIR = "tests";

	public static void main(String[] args) {
		File[] problems = new File(PROBLEMS_DIR).listFiles();
		Map<String, Set<String>> sols = new TreeMap<String, Set<String>>();
		Set<String> judges = new TreeSet<String>(); 
		for (File problem : problems) {
			if (problem.isDirectory() && new File(problem, TESTS_DIR).isDirectory()) {
				List<String> psols = listSols(problem);
				judges.addAll(psols);
				sols.put(problem.getName(), new HashSet<String>(psols));
			}
		}
		System.out.printf("%20s : ", "PROBLEM");
		for (String judge : judges) {
			System.out.printf("%3s", abbr(judge));
		}
		System.out.printf("%3s%n", "#");
		Map<String, Integer> cnt = new HashMap<String, Integer>();
		for (Map.Entry<String, Set<String>> entry : sols.entrySet()) {
			String problem = entry.getKey();
			if (problem.startsWith("a") || problem.startsWith("x")) {
				// new section
				System.out.printf("%20s : ", "------");
				for (String judge : judges) {
					System.out.printf("%3s", "--");
				}
				System.out.printf("%n");
				if (!cnt.isEmpty()) {
					System.out.printf("%20s : ", "#");
					for (String judge : judges) {
						Integer c = cnt.get(judge);
						System.out.printf("%3d", c == null ? 0 : c);
					}
					System.out.printf("%n");
				}
			}
			Set<String> psols = entry.getValue();
			System.out.printf("%20s : ", problem);
			for (String judge : judges) {
				boolean hassol = psols.contains(judge);
				System.out.printf("%3s", hassol ? "+" : "");
				if (hassol) {
					Integer c = cnt.get(judge);
					cnt.put(judge, c == null ? 1 : c + 1);
				}
			}
			System.out.printf("%3d%n", psols.size());
		}
	}

	private static Object abbr(String judge) {
		return judge.length() <= 2 ? judge : judge.substring(0, 2);
	}

	private static List<String> listSols(File problem) {
		String prefix = problem.getName() + "_";
		List<String> result = new ArrayList<String>();
		String[] names = problem.list();
		for (String name : names) {
			if (name.startsWith(prefix) && (name.endsWith(".java") || name.endsWith(".cpp") || name.endsWith(".dpr"))) {
				String sufix = name.substring(prefix.length());
				if (sufix.indexOf('_') < 0 && sufix.indexOf('.') > 0)
					result.add(sufix.substring(0, sufix.indexOf('.')));
			}
		}
		return result;
	}
}
