import java.io.*
import java.util.*
import kotlin.system.measureTimeMillis

/**
 * Solution for NEERC'2016 Problem A: Abbreviation
 * This solution requires correctness of the input.
 * @author Roman Elizarov
 */

const val INPUT = "abbreviation.in"
const val OUTPUT = "abbreviation.out"

const val MAX_LINES = 1000
const val MAX_CHARS = 120

fun main(args: Array<String>) {
    val time = measureTimeMillis {
        BufferedReader(FileReader(INPUT)).use { it.readInput() }
        solve()
        PrintWriter(OUTPUT).use { it.writeOutput() }
    }
    println("Done in $time ms")
}

// input

val input = arrayListOf<String>()

fun BufferedReader.readInput() {
    while (true) {
        val line = readLine() ?: break
        require(line.length <= MAX_CHARS) { "Line is too long ${line.length}" }
        for (c in line) when (c) {
            ' ', in 'a'..'z', in 'A'..'Z', ',', '.' -> {}
            else -> fail("Illegal character '$c'")
        }
        require(!line.isEmpty()) { "Empty line" }
        require(line.first() != ' ' && line.last() != ' ') { "Leading or trailing spaces" }
        input.add(line)
    }
    require(input.size > 0) { "At least one line "}
    require(input.size <= MAX_LINES) { "Too many lines ${input.size}"}
}

// solution

val output = arrayListOf<String>()

fun solve() {
    for (originalLine in input) {
        var line = originalLine
        var i = 0

        fun hasMore() = i < line.length
        fun cur() = line[i]

        fun next(): Char {
            val cur = cur()
            i++
            return cur
        }

        fun findWordStart() {
            while (hasMore()) when (cur()) {
                in 'a'..'z', in 'A'..'Z' -> return
                else -> next()
            }
        }

        fun skipWord() {
            while (hasMore()) when (cur()) {
                in 'a'..'z', in 'A'..'Z' -> next()
                else -> return
            }
        }

        main@ while (hasMore()) {
            findWordStart()
            val start = i
            var wc = 0
            var lastGood = 0
            val abbrev = StringBuilder()
            abbrev@ while (true) {
                if (!hasMore()) break@abbrev
                if (cur() !in 'A'..'Z') { skipWord(); break@abbrev }
                abbrev.append(cur())
                next()
                if (!hasMore()) break@abbrev
                if (cur() !in 'a'..'z') { skipWord(); break@abbrev }
                word@ while (hasMore()) when (cur()) {
                    in 'a'..'z' -> next()
                    in 'A'..'Z' -> { skipWord(); break@abbrev }
                    ' ', '.', ',' -> break@word
                    else -> fail("Cannot happen")
                }
                wc++
                if (wc >= 2)
                    lastGood = i
                if (!hasMore()) break@abbrev
                if (next() != ' ') break@abbrev
            }
            if (lastGood > 0) {
                val ab = abbrev.substring(0, wc)
                line = line.substring(0, start) + ab + " (" + line.substring(start, lastGood) + ")" + line.substring(lastGood)
                i += ab.length + 3
            }
        }

        output.add(line)
    }
}

// result

fun PrintWriter.writeOutput() {
    output.forEach { println(it) }
}

//----------------- utility ------------------

fun fail(s: String): Nothing = throw IllegalArgumentException(s)
