import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;
import java.util.stream.Stream;

/**
 * Created by Niyaz Nigmatullin on 18.11.16.
 */
public class Tests {

    static Random rng = new Random(50216239017L);
    static File testsDirectory;
    static int currentTest;

    static class Test {
        String[] w;
        String comment;

        public Test(String[] w, String comment) {
            this.w = w;
            this.comment = comment;
        }
    }

    static void printTest(Test test) {
        currentTest++;
        System.err.println("Printing test " + currentTest + ". " + test.comment);
        try {
            PrintWriter out = new PrintWriter(new File(testsDirectory, String.format("%03d", currentTest)));
            out.println(test.w.length);
            for (String e : test.w) {
                out.println(e);
            }
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static String randomBits(int length, int c0, int c1) {
        StringBuilder sb = new StringBuilder(length);
        for (int i = 0; i < length; i++) {
            sb.append(rng.nextInt(c0 + c1) < c0 ? '0' : '1');
        }
        return sb.toString();
    }

    static String setQ(String s, int pos) {
        char[] c = s.toCharArray();
        c[pos] = '?';
        return new String(c);
    }

    static String setRandomQ(String s) {
        return setQ(s, rng.nextInt(s.length()));
    }

    static Test random1(int sumLen, int length) {
        String comment = "Random strings of length up to " + length + " of total length = " + sumLen + ".";
        List<String> ret = new ArrayList<>();
        while (sumLen > 0) {
            String word = setRandomQ(randomBits(rng.nextInt(Math.min(sumLen, length)) + 1, 1, 1));
            ret.add(word);
            sumLen -= word.length();
        }
        Collections.shuffle(ret, rng);
        while (ret.size() > MAXN) {
            ret.remove(ret.size() - 1);
        }
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static Test random2(int len) {
        String comment = "Random test of total length " + len + ".";
        List<String> ret = new ArrayList<>();
        while (len > 3) {
            String word = randomBits(Math.min(len, rng.nextInt(rng.nextInt(len) + 3) + 3), 1, 1);
            ret.add(word);
            len -= word.length();
        }
        if (len > 0) {
            ret.add(randomBits(len, 1, 1));
        }
        int prob = rng.nextInt(100) + 1;
        for (int i = 0; i < ret.size(); i++) {
            if (rng.nextInt(100) < prob) {
                ret.set(i, setRandomQ(ret.get(i)));
            }
        }
        Collections.shuffle(ret, rng);
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static Test random3(int len) {
        String comment = "Random test of total length " + len + ", solutions exists.";
        List<String> ret = new ArrayList<>();
        int cc = 0;
        while (len > 0) {
            ++cc;
            loop: while (true) {
                int curLen;
                if (cc <= 2) {
                    curLen = len > 10 ? rng.nextInt(rng.nextInt(len - 10) + 1) + 5 : len;
                } else {
                    curLen = len > 40 ? rng.nextInt(Math.min(len - 40, 1000) + 17) : len;
                }
                String word = randomBits(curLen, 1, 1);
                for (String e : ret) {
                    if (e.startsWith(word) || word.startsWith(e)) {
                        continue loop;
                    }
                }
                ret.add(word);
                len -= word.length();
                break;
            }
        }
        int prob = rng.nextInt(100) + 1;
        for (int i = 0; i < ret.size(); i++) {
            if (rng.nextInt(100) < prob) {
                ret.set(i, setRandomQ(ret.get(i)));
            }
        }
        Collections.shuffle(ret, rng);
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static Test special1(int count) {
        String comment = "Here are " + count + " strings of length " + count + " differing in question mark position.";
        String f = randomBits(count, 1, 1);
        List<String> ret = new ArrayList<>();
        for (int i = 0; i < count; i++) {
            ret.add(setQ(f, i));
        }
        Collections.shuffle(ret, rng);
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static String flip(String s, int pos) {
        char[] c = s.toCharArray();
        c[pos] = c[pos] == '0' ? '1' : '0';
        return new String(c);
    }

    static Test special2(int length, int different, int count) {
        String comment = "special2 " + (length + " " + different + " " + count);
        if (count > different * length) throw new AssertionError("count > different * length");
        if (length <= 30 && count > 1 << length) throw new AssertionError("count > 2^(length-1)");
        List<String> f = new ArrayList<>();
        f.add(randomBits(length, 1, 1));
        int[] posQ = new int[count];
        Set<String> q = new HashSet<>();
        q.add(f.get(0));
        for (int i = 1; i < different; i++) {
            String en;
            do {
                String e = f.get(rng.nextInt(f.size()));
                int pos = rng.nextInt(e.length());
                en = flip(e, pos);
                posQ[i] = pos;
            } while (!q.add(en));
            f.add(en);
        }
        boolean[][] made = new boolean[different][length];
        for (int i = 0; i < different; i++) {
            made[i][posQ[i]] = true;
        }
        for (int i = different; i < count; i++) {
            int cc = 0;
            all: while (true) {
                if (++cc == 50) {
                    for (int j = 0; j < different; j++) {
                        for (int k = 0; k < length; k++) {
                            if (!made[j][k]) {
                                made[j][k] = true;
                                String e = flip(f.get(j), k);
                                if (q.add(e)) {
                                    posQ[f.size()] = k;
                                    f.add(e);
                                    break all;
                                }
                            }
                        }
                    }
                    return null;
                }
                int which = rng.nextInt(different);
                int pos = rng.nextInt(length);
                if (made[which][pos]) continue;
                made[which][pos] = true;
                String e = flip(f.get(which), pos);
                if (q.add(e)) {
                    posQ[f.size()] = pos;
                    f.add(e);
                    break;
                }
            }
        }
        int sumLen = 0;
        for (String e : f) sumLen += e.length();
        all: for (int it = 0; it < 1000 && sumLen < MAXS; it++) {
            String e = f.get(rng.nextInt(f.size()));
            e = e.substring(0, Math.min(rng.nextInt(Math.min(15, MAXS - sumLen)) + 1, e.length()));
            e = flip(e, rng.nextInt(e.length()));
            for (String r : f) {
                if (r.startsWith(e) || e.startsWith(r)) {
                    continue all;
                }
            }
            f.add(e);
        }
        List<String> ret = new ArrayList<>();
        for (int i = 0; i < f.size(); i++) {
            if (i < count) {
                ret.add(setQ(f.get(i), posQ[i]));
            } else {
                ret.add(setRandomQ(f.get(i)));
            }
        }
        Collections.shuffle(ret, rng);
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static final int MAXS = 500000;
    static final int MAXN = 500000;

    static Test cut(Test test, int count) {
        String comment = test.comment + " Some strings are removed.";
        List<String> ret = new ArrayList<>(Arrays.asList(test.w));
        Collections.shuffle(ret, rng);
        while (count > 0) {
            ret.remove(ret.size() - 1);
            --count;
        }
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static Test prependString(Test test) {
        int sum = Stream.of(test.w).mapToInt(String::length).sum();
        int n = test.w.length;
        int toAdd = (MAXS - sum) / n;
        String s = randomBits(toAdd, 1, 1);
        return new Test(Stream.of(test.w).map(x -> s + x).toArray(String[]::new), test.comment + " Prepended same random string.");
    }

    static void generateTests() {
        // sample test cases
        printTest(new Test(new String[]{"00?", "0?00", "?1", "1?0"}, "sample test 1"));
        printTest(new Test(new String[]{"0100", "01?0", "01?0"}, "sample test 2"));
        // main test data
        for (int length = 10; length <= MAXS; length += 4 * length / 5) {
            printTest(random2(length));
            printTest(random3(length));
        }
        printTest(random2(MAXS));
        printTest(random3(MAXS));
        for (int i = 2; i <= 20; i++) {
            printTest(all(i));
        }
        printTest(prependString(all(10)));
        printTest(prependString(all(8)));
        printTest(prependString(all(12)));
        printTest(maxNumberOfStrings(MAXS));
        printTest(cut(maxNumberOfStrings(MAXS), 100));
        printTest(random1(MAXS, 2));
        printTest(random1(MAXS, 3));
        printTest(random1(MAXN, 1));
        printTest(random1(MAXS, 10));
        for (int length : new int[]{444, 123456, MAXS}) {
            printTest(special3(length));
            printTest(noSolution(special3(length)));
        }
        for (int count = 20; count * count <= MAXS; count += 164) {
            printTest(special1(count));
        }
        printTest(new Test(new String[]{randomBits(MAXS, 1, 1)}, "One string of length " + MAXS));
        printTest(new Test(new String[]{randomBits(MAXS / 2, 1, 1), randomBits(MAXS / 2, 1, 1)},
                            "Two strings of length " + (MAXS / 2)));
        for (int count = 20; count * 10 <= MAXS; count += 2777) {
            int cc = 0;
            while (true) {
                if (++cc == 10000) {
                    count = MAXS;
                    break;
                }
                int maxLen = MAXS / count;
                int len = rng.nextInt(maxLen - maxLen / 2) + maxLen / 2;
                int different = rng.nextInt(count - 4) + 5;
                while (different * len < 2 * count) ++different;
                if (different * len < count || different > count || len <= 30 && count > 1 << len - 1) continue;
                Test got = special2(len, different, count);
                if (got == null) {
                    continue;
                }
                printTest(got);
                if (rng.nextInt(4) == 0) {
                    printTest(noSolution(got));
                }
                break;
            }
        }
//        printTest(special2(16, 9001, 40000));
//        printTest(special2(17, 15000, 40000));
//        printTest(noSolution(special2(17, 15000, 40000)));
//        printTest(special2(16, 20000, 60000));
        printTest(special2(15, 4501, 20000));
        printTest(special2(16, 6510, 20000));
        printTest(noSolution(special2(16, 6500, 20100)));
        {
            int dif = 9000;
            int s = 30021;
            Test test;
            while ((test = special2(15, dif, s)) == null) {
                if (rng.nextBoolean()) {
                    dif += 2;
                } else {
                    s -= 2;
                }
            }
            printTest(test);
        }
    }

    static Test noSolution(Test test) {
        String comment = test.comment + " No solution.";
        List<String> ret = new ArrayList<>();
        for (String e : test.w) {
            ret.add(e);
        }
        int change = rng.nextInt(10) + 5;
        for (int i = 0; i < change; i++) {
            int id = rng.nextInt(ret.size());
            String e = ret.get(id);
            ret.set(id, e.replace('?', (char) ('0' + rng.nextInt(2))));
        }
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static Test maxNumberOfStrings(int sumLen) {
        String comment = "Maximum number of strings for total length = " + sumLen + ".";
        int bits = 0;
        while ((1 << (bits + 1)) * (bits + 1) <= sumLen) ++bits;
        List<String> all = new ArrayList<>();
        for (int i = 0; i < 1 << bits; i++) {
            StringBuilder sb = new StringBuilder(bits);
            for (int j = 0; j < bits; j++) {
                sb.append((i >> j) & 1);
            }
            all.add(setRandomQ(sb.toString()));
        }
        boolean[] used = new boolean[1 << bits];
        List<String> ret = new ArrayList<>();
        int totalLength = (1 << bits) * bits;
        while (totalLength - bits + 2 * (bits + 1) <= sumLen) {
            totalLength = totalLength - bits + 2 * (bits + 1);
            int id;
            do {
                id = rng.nextInt(1 << bits);
            } while (used[id]);
            used[id] = true;
            ret.add(all.get(id) + '0');
            ret.add(all.get(id) + '1');
        }
        for (int i = 0; i < 1 << bits; i++) {
            if (!used[i]) ret.add(all.get(i));
        }
        Collections.shuffle(ret, rng);
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static Test special3(int sumLen) {
        String comment = "All strings are prefix of some randomly generated string. Total length = " + sumLen + ".";
        String e = randomBits(100000, 1, 1);
        List<String> ret = new ArrayList<>();
        for (int length = 5; length <= sumLen; length++) {
            for (int pos = length - 1; length <= sumLen && pos < length; pos++) {
                ret.add(setQ(e.substring(0, length), pos));
                sumLen -= length;
            }
        }
        Collections.shuffle(ret, rng);
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static Test all(int bits) {
        String comment = "All binary strings of len " + bits + ".";
        List<String> ret = new ArrayList<>();
        for (int i = 0; i < 1 << bits; i++) {
            StringBuilder sb = new StringBuilder(bits);
            for (int j = 0; j < bits; j++) {
                sb.append((i >> j) & 1);
            }
            ret.add(setRandomQ(sb.toString()));
        }
        Collections.shuffle(ret, rng);
        int sumLen = (1 << bits) * bits;
        while (sumLen > MAXS) {
            String removed = ret.remove(ret.size() - 1);
            sumLen -= removed.length();
        }
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    static Test all(int fromBits, int bits) {
        String comment = "All binary strings of length from " + fromBits + " to " + bits + ".";
        List<String> ret = new ArrayList<>();
        boolean[] used = new boolean[1 << bits];
        for (int i = 0; i < 1 << bits; i++) {
            int curBits = rng.nextInt(bits - fromBits + 1) + fromBits;
            int mask = rng.nextInt(1 << curBits);
            if (used[mask]) continue;
            for (int addMask = 0; addMask < 1 << (bits - curBits); addMask++) {
                used[mask | (addMask << curBits)] = true;
            }
            StringBuilder sb = new StringBuilder(bits);
            for (int j = 0; j < curBits; j++) {
                sb.append((mask >> j) & 1);
            }
            ret.add(setRandomQ(sb.toString()));
        }
        for (int i = (1 << bits) - 1; i >= 0; i--) {
            if (Integer.bitCount(i) < fromBits) continue;
            if (used[i]) continue;
            StringBuilder sb = new StringBuilder(bits);
            for (int j = 0; j < bits; j++) {
                sb.append((i >> j) & 1);
            }
            ret.add(setRandomQ(sb.toString()));
        }
        Collections.shuffle(ret, rng);
        int sumLen = 0;
        for (String e : ret) sumLen += e.length();
        while (sumLen > MAXS) {
            String removed = ret.remove(ret.size() - 1);
            sumLen -= removed.length();
        }
        return new Test(ret.toArray(new String[ret.size()]), comment);
    }

    public static void main(String[] args) {
        testsDirectory = new File("tests");
        if (!testsDirectory.exists()) {
            if (!testsDirectory.mkdirs()) {
                System.err.println("couldn't create tests directory");
                System.exit(1);
            }
        }
        currentTest = 0;
        generateTests();
    }
}
