#ifdef DBG1
  #define _GLIBCXX_DEBUG
#endif

#include <cstdio>
#include <cassert>
#include <cstring>
#include <string>
#include <vector>
#include <algorithm>
#include <map>
#include <set>

#ifdef DBG1
    #define eprintf(...) fprintf(stderr, __VA_ARGS__)
#else
    #define eprintf(...)
#endif

using namespace std;

typedef long long ll;
typedef unsigned long long ull;
typedef pair <int, int> pii;

#define TASKNAME "binary"

class WordComparator {
  const vector <char*> & words;
public:
  WordComparator(const vector <char*> words) : words(words) {}

  bool operator () (int i1, int i2) {
    return strcmp(words[i1], words[i2]) < 0;
  }
};

bool isPrefix(const char *s1, const char *s2) {
  for (int i = 0; s1[i]; ++i) {
    if (s1[i] != s2[i]) {
      return false;
    }
  }
  return true;
}

int main() {
  freopen(TASKNAME ".in", "r", stdin);
  freopen(TASKNAME ".out", "w", stdout);

  static char text[2 * int(5e5)];
  char *text_ptr = text;

  int n;
  scanf("%d", &n);

  vector <char*> s(n);
  vector <int> pos(n);
  for (int i = 0; i < n; ++i) {
    s[i] = text_ptr;
    scanf("%s", s[i]);
    int len = strlen(s[i]);
    text_ptr = s[i] + len + 1;

    char *curPos = strchr(s[i], '?');
    pos[i] = (curPos ? curPos - s[i] : -1);
  }

  vector <int> ind(n);
  for (int i = 0; i < n; ++i) {
    ind[i] = i;
  }

  sort(ind.begin(), ind.end(), WordComparator(s));
  for (int i = 1; i < n; ++i) {
    int i1 = ind[i - 1];
    int i2 = ind[i];
    if (isPrefix(s[i1], s[i2])) {
      if (pos[i1] != -1 && pos[i2] != -1) {
        assert(pos[i1] == pos[i2]);
        s[i1][pos[i1]] = '0';
        s[i2][pos[i2]] = '1';
      }
    }
  }
  sort(ind.begin(), ind.end(), WordComparator(s));
  for (int i = 1; i < n; ++i) {
    if (isPrefix(s[ind[i - 1]], s[ind[i]])) {
      printf("NO\n");
      return 0;
    }
  }

  printf("YES\n");
  for (int i = 0; i < n; ++i) {
    printf("%s\n", s[i]);
  }

  return 0;
}
