import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by Niyaz Nigmatullin on 20.11.16.
 */
public class binary_nn_Sn {

    static class TwoSATSolver {
        List<Integer>[] edges, rEdges;
        int varsCount;

        public TwoSATSolver(int n) {
            this.varsCount = n;
            edges = new List[2 * n];
            rEdges = new List[2 * n];
            for (int i = 0; i < 2 * n; i++) {
                edges[i] = new ArrayList<>();
                rEdges[i] = new ArrayList<>();
            }
        }

        int not(int v) {
            return v < varsCount ? v + varsCount : v - varsCount;
        }

        private void addEdge(int v, int u) {
            edges[v].add(u);
            rEdges[u].add(v);
        }

        void addClause(int v, int u) {
            v = v < 0 ? (~v) + varsCount : v;
            u = u < 0 ? (~u) + varsCount : u;
            addEdge(not(v), u);
            addEdge(not(u), v);
        }

        boolean[] was;
        int[] order;
        int cn;
        int[] color;

        private void makeOrder(int v) {
            was[v] = true;
            for (int i = 0; i < edges[v].size(); i++) {
                int to = edges[v].get(i);
                if (!was[to]) {
                    makeOrder(to);
                }
            }
            order[cn++] = v;
        }

        private void makeColor(int v, int c) {
            color[v] = c;
            for (int i = 0; i < rEdges[v].size(); i++) {
                int to = rEdges[v].get(i);
                if (color[to] < 0) {
                    makeColor(to, c);
                }
            }
        }

        boolean[] solve() {
            int n = edges.length;
            cn = 0;
            order = new int[n];
            was = new boolean[n];
            for (int i = 0; i < n; i++) {
                if (was[i]) continue;
                makeOrder(i);
            }
            int colors = 0;
            color = new int[n];
            Arrays.fill(color, -1);
            for (int it = cn - 1; it >= 0; it--) {
                int v = order[it];
                if (color[v] < 0) {
                    makeColor(v, colors++);
                }
            }
            for (int i = 0; i < varsCount; i++) {
                if (color[i] == color[not(i)]) {
                    return null;
                }
            }
            int[] backColor = new int[colors];
            Arrays.fill(backColor, -1);
            for (int i = 0; i < n; i++) {
                int c = color[i];
                int cc = color[not(i)];
                if (backColor[c] >= 0 && backColor[c] != cc) {
                    throw new AssertionError();
                }
                backColor[c] = cc;
            }
            if ((colors & 1) == 1) throw new AssertionError();
            int[] w = new int[colors];
            Arrays.fill(w, -1);
            for (int i = colors - 1; i >= 0; i--) {
                if (w[i] < 0) {
                    w[i] = 1;
                    w[backColor[i]] = 0;
                }
            }
            boolean[] ret = new boolean[varsCount];
            for (int i = 0; i < varsCount; i++) {
                ret[i] = w[color[i]] == 1;
            }
            return ret;
        }
    }

    static String[] solveSn(String[] codeWords) { // x_i = true, if replaced by '0'
        int n = codeWords.length;
        TwoSATSolver sat = new TwoSATSolver(n);
        for (int i = 0; i < n; i++) {
            for (int q1 = 0; q1 < 2; q1++) {
                String word1 = codeWords[i].replace('?', (char) (q1 + '0'));
                int var1 = q1 == 0 ? i : ~i;
                for (int j = 0; j < n; j++) {
                    if (i == j) continue;
                    for (int q2 = 0; q2 < 2; q2++) {
                        String word2 = codeWords[j].replace('?', (char) (q2 + '0'));
                        int var2 = q2 == 0 ? j : ~j;
                        if (word1.startsWith(word2) || word2.startsWith(word1)) {
                            sat.addClause(~var1, ~var2);
                        }
                    }
                }
            }
        }
        boolean[] got = sat.solve();
        if (got == null) {
            return null;
        }
        String[] ans = new String[n];
        for (int i = 0; i < n; i++) {
            ans[i] = codeWords[i].replace('?', got[i] ? '0' : '1');
        }
        return ans;
    }

    static void solve() throws IOException {
        int n = Integer.parseInt(br.readLine());
        String[] s = new String[n];
        for (int i = 0; i < n; i++) {
            s[i] = br.readLine();
        }
        String[] ans = solveSn(s);
        if (ans == null) {
            out.println("NO");
        } else {
            out.println("YES");
            for (String e : ans) {
                out.println(e);
            }
        }
    }

    static BufferedReader br;
    static PrintWriter out;

    public static void main(String[] args) throws IOException {
        br = new BufferedReader(new FileReader("binary.in"));
        out = new PrintWriter("binary.out");
        solve();
        out.close();
        br.close();
    }

}
