import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by Niyaz Nigmatullin on 20.11.16.
 */
public class binary_nn_java {

    static class TwoSATSolver {
        int[] fromEdges;
        int[] toEdges;
        int varsCount;
        int edgesCount;
        int[] head, next;
        int[] headR, nextR;

        public TwoSATSolver(int n, int sumLen) {
            this.varsCount = n;
            fromEdges = new int[(1 + sumLen) * 2];
            toEdges = new int[(1 + sumLen) * 2];
            edgesCount = 0;
        }

        int not(int v) {
            return v < varsCount ? v + varsCount : v - varsCount;
        }

        private void addEdge(int v, int u) {
            if (edgesCount == fromEdges.length) {
                fromEdges = Arrays.copyOf(fromEdges, fromEdges.length * 2);
                toEdges = Arrays.copyOf(toEdges, toEdges.length * 2);
            }
            fromEdges[edgesCount] = v;
            toEdges[edgesCount] = u;
            ++edgesCount;
        }

        void addClause(int v, int u) {
            v = v < 0 ? (~v) + varsCount : v;
            u = u < 0 ? (~u) + varsCount : u;
            addEdge(not(v), u);
            addEdge(not(u), v);
        }

        boolean[] was;
        int[] order;
        int cn;
        int[] color;

        private void makeOrder(int v) {
            was[v] = true;
            for (int e = head[v]; e >= 0; e = next[e]) {
                int to = toEdges[e];
                if (!was[to]) {
                    makeOrder(to);
                }
            }
            order[cn++] = v;
        }

        private void makeColor(int v, int c) {
            color[v] = c;
            for (int e = headR[v]; e >= 0; e = nextR[e]) {
                int to = fromEdges[e];
                if (color[to] < 0) {
                    makeColor(to, c);
                }
            }
        }

        boolean[] solve() {
            int n = varsCount * 2;
            head = new int[n];
            headR = new int[n];
            for (int i = 0; i < n; i++) {
                head[i] = headR[i] = -1;
            }
            next = new int[edgesCount];
            nextR = new int[edgesCount];
            for (int i = 0; i < edgesCount; i++) {
                next[i] = head[fromEdges[i]];
                head[fromEdges[i]] = i;
                nextR[i] = headR[toEdges[i]];
                headR[toEdges[i]] = i;
            }
            cn = 0;
            order = new int[n];
            was = new boolean[n];
            for (int i = 0; i < n; i++) {
                if (was[i]) continue;
                makeOrder(i);
            }
            int colors = 0;
            color = new int[n];
            Arrays.fill(color, -1);
            for (int it = cn - 1; it >= 0; it--) {
                int v = order[it];
                if (color[v] < 0) {
                    makeColor(v, colors++);
                }
            }
            for (int i = 0; i < varsCount; i++) {
                if (color[i] == color[not(i)]) {
                    return null;
                }
            }
            int[] backColor = new int[colors];
            Arrays.fill(backColor, -1);
            for (int i = 0; i < n; i++) {
                int c = color[i];
                int cc = color[not(i)];
                if (backColor[c] >= 0 && backColor[c] != cc) {
                    throw new AssertionError();
                }
                backColor[c] = cc;
            }
            if ((colors & 1) == 1) throw new AssertionError();
            int[] w = new int[colors];
            Arrays.fill(w, -1);
            for (int i = colors - 1; i >= 0; i--) {
                if (w[i] < 0) {
                    w[i] = 1;
                    w[backColor[i]] = 0;
                }
            }
            boolean[] ret = new boolean[varsCount];
            for (int i = 0; i < varsCount; i++) {
                ret[i] = w[color[i]] == 1;
            }
            return ret;
        }
    }

    static List<Integer>[] entries;
    static int[] posQ;

    static String[] solve(String[] codeWords) { // x_i = true, if replaced by '0'
        int n = codeWords.length;
        if (n > 34000) {
            return null;
        }
        Word[] words = new Word[2 * n];
        int all = 1;
        posQ = new int[n];
        for (int i = 0; i < n; i++) {
            words[2 * i] = new Word(codeWords[i].replace('?', '0'), i);
            words[2 * i + 1] = new Word(codeWords[i].replace('?', '1'), ~i);
            posQ[i] = codeWords[i].indexOf('?');
            all += codeWords[i].length() * 2;
        }
        link0 = new int[all];
        link1 = new int[all];
        entries = new List[all];
        Arrays.fill(link0, -1);
        Arrays.fill(link1, -1);
        int fr = 0;
        int root = fr++;
        for (int i = 0; i < n; i++) {
            for (int k = 0; k < 2; k++) {
                int v = root;
                for (char c : codeWords[i].toCharArray()) {
                    int letter = c == '?' ? k : (c - '0');
                    int[] link = (letter == 0 ? link0 : link1);
                    if (link[v] < 0) {
                        link[v] = fr++;
                    }
                    v = link[v];
                }
                if (entries[v] == null) {
                    entries[v] = new ArrayList<>();
                }
                entries[v].add(k == 0 ? i : ~i);
            }
        }
        stack = new int[2 * n];
        cn = 0;
        sat = new TwoSATSolver(n, all);
        if (!dfs(0)) {
            return null;
        }
        System.err.println(sat.edgesCount);
        boolean[] got = sat.solve();
        if (got == null) {
            return null;
        }
        String[] ans = new String[n];
        for (int i = 0; i < n; i++) {
            ans[i] = codeWords[i].replace('?', got[i] ? '0' : '1');
        }
        return ans;
    }

    static boolean dfs(int v) {
        int oldCn = cn;
        if (entries[v] != null) {
            for (int it = 0; it < entries[v].size(); it++) {
                int who = -1;
                int cur = entries[v].get(it);
                int id = cur < 0 ? ~cur : cur;
                for (int i = 0; i < cn; i++) {
                    int parent = stack[i];
                    if (cur == parent || ~cur == parent) continue;
                    sat.addClause(~cur, ~parent);
                    int jd = parent < 0 ? ~parent : parent;
                    if (posQ[id] >= 0 && posQ[id] == posQ[jd]) {
                        if (who >= 0 && who != jd) {
                            return false;
                        }
                        who = jd;
                    }
                }
                stack[cn++] = entries[v].get(it);
            }
        }
        if (link0[v] >= 0) {
            if (!dfs(link0[v])) return false;
        }
        if (link1[v] >= 0) {
            if (!dfs(link1[v])) return false;
        }
        cn = oldCn;
        return true;
    }

    static int cn;
    static int[] stack;
    static TwoSATSolver sat;
    static int[] link0, link1;

    static void solve() throws IOException {
        int n = Integer.parseInt(br.readLine());
        String[] s = new String[n];
        for (int i = 0; i < n; i++) {
            s[i] = br.readLine();
        }
        String[] ans = solve(s);
        if (ans == null) {
            out.println("NO");
        } else {
            out.println("YES");
            for (String e : ans) {
                out.println(e);
            }
        }
    }

    static class Word {
        String s;
        int id;

        public Word(String s, int id) {
            this.s = s;
            this.id = id;
        }
    }

    static BufferedReader br;
    static PrintWriter out;

    public static void main(String[] args) throws IOException {
        br = new BufferedReader(new FileReader("binary.in"));
        out = new PrintWriter("binary.out");
        solve();
        out.close();
        br.close();
    }

}
