//#include <iostream>
#include <fstream>
#include <vector>
#include <set>
#include <map>
#include <cstring>
#include <string>
#include <cmath>
#include <cassert>
#include <ctime>
#include <algorithm>
#include <sstream>
#include <list>
#include <queue>
#include <deque>
#include <stack>
#include <cstdlib>
#include <cstdio>
#include <iterator>
#include <functional>
#include <bitset>
#define mp make_pair
#define pb push_back

#ifdef LOCAL
#define eprintf(...) fprintf(stderr,__VA_ARGS__)
#else
#define eprintf(...)
#endif

#define TIMESTAMP(x) eprintf("["#x"] Time : %.3lf s.\n", clock()*1.0/CLOCKS_PER_SEC)
#define TIMESTAMPf(x,...) eprintf("[" x "] Time : %.3lf s.\n", __VA_ARGS__, clock()*1.0/CLOCKS_PER_SEC)

#if ( ( _WIN32 || __WIN32__ ) && __cplusplus < 201103L)
    #define LLD "%I64d"
#else
    #define LLD "%lld"
#endif

using namespace std;

#define TASKNAME "cactus"

#ifdef LOCAL
static struct __timestamper {
    string what;
    __timestamper(const char* what) : what(what){};
    __timestamper(const string& what) : what(what){};
	~__timestamper(){
        TIMESTAMPf("%s", what.data());
	}
} __TIMESTAMPER("end");
#else 
struct __timestamper {};
#endif

typedef long long ll;
typedef long double ld;


vector<vector<int>> g;
vector<vector<int>> cg;
int n, m;

vector<pair<int, int>> st;
vector<int> tin;
vector<int> tup;
int timer;

void dfs0(int v, int p) {
    tin[v] = tup[v] = timer++;
    for (int u : g[v]) {
        if (u == p) continue;
        if (tin[u] > tin[v]) continue;
        st.push_back(make_pair(v, u));
        if (tin[u] != -1) {
            tup[v] = min(tup[v], tin[u]);
            continue;
        }
        dfs0(u, v);
        tup[v] = min(tup[v], tup[u]);
        if (tup[u] >= tin[v]) {
            vector<int> cycle;
            cycle.push_back(v);
            while (true) {
                auto x = st.back();
                st.pop_back();
                if (x == make_pair(v, u)) {
                    break;
                }
                cycle.push_back(x.first);
            }
            if (cycle.size() == 1) {
                cycle.push_back(u);
            }
            if (cycle.size() == 2) {
                cg[cycle[0]].push_back(cycle[1]);
                cg[cycle[1]].push_back(cycle[0]);
            } else {
                int cid = cg.size();
                cg.push_back(cycle);
                for (int t : cycle) {
                    cg[t].push_back(cid);
                }
            }
        }
    }
}

struct op {
  int type;
  int a, b, c;
};

vector<op> ans;

void build(int v, int p = -1) {
  for (int u : cg[v]) {
    if (u != p) {
      build(u, v);
    }
  }
  if (v < n) {
    ans.push_back({1, v, 1, 2});
    for (int u : cg[v]) {
      if (u < n && u != p) {
        ans.push_back({0, v, u, 0}); 
      }
    }
    ans.push_back({2, v, 1, 2});
    ans.push_back({1, v, 1, 4});
    ans.push_back({1, v, 2, 1});
  } else {
    if (p != -1) {
      int id = find(cg[v].begin(), cg[v].end(), p) - cg[v].begin();
      assert(id != cg[v].size());
      rotate(cg[v].begin(), cg[v].begin() + id, cg[v].end());
    }
    int last = 1;
    for (int i = 1; i < (int)cg[v].size(); i++) {
      int cur = (last == 2) ? 3 : 2;
      ans.push_back({1, cg[v][i], 1, cur});
      ans.push_back({0, cg[v][i], cg[v][i-1], 0});
      ans.push_back({2, cg[v][i], last, cur});
      if (i != 1) {
        ans.push_back({1, cg[v][i], last, 4});
      }
      last = cur;
    }
    ans.push_back({2, cg[v][0], 1, last});
    ans.push_back({1, cg[v][0], last, 4});
  }
}

int main() {
    assert(freopen(TASKNAME".in", "r", stdin));
    assert(freopen(TASKNAME".out", "w", stdout));
    assert(scanf("%d%d", &n, &m) == 2);
    g.resize(n);
    for (int i = 0; i < m; i++) {
        int k;
        assert(scanf("%d", &k) == 1);
        int prev;
        assert(scanf("%d", &prev) == 1);
        --prev;
        for (int j = 1; j < k; j++) {
            int next;
            assert(scanf("%d", &next) == 1);
            --next;
            g[prev].push_back(next);
            g[next].push_back(prev);
            prev = next;
        }
    }
    tin.resize(n, -1);
    tup.resize(n, -1);
    cg.resize(n);
    dfs0(0, -1);

    build(0);
    printf("%d\n", (int)ans.size());
    for (auto x : ans) {
      if (x.type == 0) {
        printf("j %d %d\n", x.a + 1, x.b + 1);
      } else if (x.type == 1) {
        printf("r %d %d %d\n", x.a + 1, x.b, x.c);
      } else if (x.type == 2) {
        printf("c %d %d %d\n", x.a + 1, x.b, x.c);
      }
    }
    return 0;
}
