import java.io.*;
import java.util.*;

public class Tests {
    private static File dir = new File("tests");
    private int curTest = 1;
    private static boolean verbose = true;

    private String testName;
    private PrintWriter out;
    private Random r = new Random(345345678678012L);
    
    public static final int MAX_DEL = 1_000_000_000;
    public static final int MAX_N = 1000;


    public void run() {
        printSample();
        for (int n = 1; n <= 2; n++) {
            for (int k = 1; k <= n; k++) {
                for (int minS = 0; minS <= k; minS++) {
                    for (int minE = 0; minS + minE <= k; minE++) {
                        printRandom(n, k, minS, minE);
                    }
                }
            }
        }
        for (int n = 3; n <= 1000; ) {
            int k = r.nextInt(n) + 1;
            int minS, minE;
            do {
                minS = r.nextInt(k + 1);
                minE = r.nextInt(k + 1);
            } while (minS + minE > k);
            printRandom(n, k, minS, minE);
            if (n < 10) {
                n++;
                continue;
            }
            if (n < 100) {
                n += 10;
                continue;
            }
            n += 100;
        }
        for (int i = 0; i < 20; i++) {
            int n = r.nextInt(MAX_N) + 1;
            int k = r.nextInt(n) + 1;
            int minS = r.nextInt(k + 1);
            int minE = r.nextInt(k - minS + 1);
            printRandom(n, k, minS, minE);
        }
        for (int i = 0; i < 20; i++) {
            int n = MAX_N;
            int k = r.nextInt(n) + 1;
            int minS = r.nextInt(k + 1);
            int minE = r.nextInt(k - minS + 1);
            printRandom(n, k, minS, minE);
        }
        printRandom(MAX_N, MAX_N, 0, 0);
        printRandom(MAX_N, MAX_N, MAX_N, 0);
        printRandom(MAX_N, MAX_N, 0, MAX_N);
        {
            int tmp = r.nextInt(MAX_N + 1);
            printRandom(MAX_N, MAX_N, tmp, MAX_N - tmp);
            printRandom(MAX_N, MAX_N, tmp, MAX_N - tmp, -1);
        }
        printRandom(MAX_N, 1, 0, 0);
        printRandom(MAX_N, 1, 1, 0);
        printRandom(MAX_N, 1, 0, 1);
        for (int diff = 0; diff < 5; diff++) {
            for (int i = 0; i < 3; i++) {
                int k = r.nextInt(MAX_N) + 1;
                if (k < diff) {
                    i--;
                    continue;
                }
                int minS, minE;
                do {
                    minS = r.nextInt(k + 1);
                    minE = r.nextInt(k + 1);
                } while (minS + minE != k - diff);
                printRandom(MAX_N, k, minS, minE, i);
            }
        }
        for (int diff = 0; diff < 5; diff++) {
            for (int i = 0; i < 3; i++) {
                int k = r.nextInt(MAX_N) + 1;
                if (k < diff) {
                    i--;
                    continue;
                }
                int minS, minE;
                do {
                    minS = r.nextInt(k + 1);
                    minE = r.nextInt(k + 1);
                } while (minS + minE != diff);
                printRandom(MAX_N, k, minS, minE, i);
            }
        }
    }

    private void printSample() {
        open();
        out.println("10 4 1 2");
        out.println("1 2 3 4 5 6 7 8 9 10");
        out.println("10 9 8 7 6 5 4 3 2 1");
        close();
    }

    private void printRandom(int n, int k, int minS, int minE) {
        printRandom(n, k, minS, minE, r.nextInt(5));
    }

    private void printRandom(int n, int k, int minS, int minE, int type) {
        open();
        int[] s = new int[n];
        int[] e = new int[n];
        for (int i = 0; i < n; i++) {
            if (type == -1) {
                s[i] = e[i] = MAX_DEL;
                continue;
            }
            if (type == 0) {
                do {
                    s[i] = r.nextInt(MAX_DEL + 1);
                    e[i] = r.nextInt(MAX_DEL + 1);
                } while (s[i] > e[i]);
            } else {
                if (type == 1) {
                    do {
                        s[i] = r.nextInt(MAX_DEL + 1);
                        e[i] = r.nextInt(MAX_DEL + 1);
                    } while (s[i] < e[i]);
                } else {
                    s[i] = r.nextInt(MAX_DEL + 1);
                    e[i] = r.nextInt(MAX_DEL + 1);
                }
            }
        }
        out.println(n + " " + k + " " + minS + " " + minE);
        for (int i = 0; i < n; i++) {
            if (i > 0) out.print(" ");
            out.print(s[i]);
        }
        out.println();
        for (int i = 0; i < n; i++) {
            if (i > 0) out.print(" ");
            out.print(e[i]);
        }
        out.println();
        close();
    }

    private void open() {
        try {
            testName = "" + curTest / 100 + curTest / 10 % 10 + curTest % 10;
            out = new PrintWriter(new File(dir, testName));
            if (verbose) { 
                System.out.print("Generating " + testName + "...");
            }
        } catch (IOException e) {
            throw new RuntimeException("Unable to open " + testName + " for writing", e);
        }
    }

    private void close() {
        out.close();
        if (verbose) {
            System.out.println(" done.");
        }
        curTest++;
    }

    public static void main(String[] args) {
        Locale.setDefault(Locale.US);
        dir.mkdirs();
        new Tests().run();
    }
}
