import java.util.*;
import java.io.*;

public class delight_av_tl2 {
	
	final long INF = Long.MAX_VALUE;

	class MCMFGraph {
		class Edge {
			int from, to, flow, cap, cost;
			Edge rev;

			public Edge(int from, int to, int cap, int cost) {
				this.from = from;
				this.to = to;
				this.cap = cap;
				this.cost = cost;
			}

			@Override
			public String toString() {
				return "Edge [from=" + from + ", to=" + to + ", flow=" + flow + ", cap=" + cap + ", cost=" + cost + "]";
			}
		}

		List<Edge>[] graph;

		public MCMFGraph(int n) {
			graph = new List[n];
			for (int i = 0; i < n; i++) {
				graph[i] = new ArrayList<>();
			}
		}

		Edge addEdge(int from, int to, int cap, int cost) {
			Edge st = new Edge(from, to, cap, cost);
			Edge rev = new Edge(to, from, 0, -cost);
			st.rev = rev;
			rev.rev = st;
			graph[from].add(st);
			graph[to].add(rev);
			return st;
		}

		void minCostFlow() {
			int n = graph.length;
			long[] h = getPotentials();
			long[] dist = new long[n];
			Edge[] prev = new Edge[n];
			TreeSet<Integer> set = new TreeSet<>((x, y) -> {
				if (dist[x] != dist[y]) {
					return Long.compare(dist[x], dist[y]);
				}
				return Integer.compare(x, y);
			});
			while (true) {
				Arrays.fill(dist, INF);
				dist[0] = 0;
				set.clear();
				set.add(0);
				while (!set.isEmpty()) {
					int cur = set.pollFirst();
					for (Edge e : graph[cur]) {
						if (e.flow < e.cap && dist[e.to] > dist[cur] + e.cost + h[cur] - h[e.from]) {
							set.remove(e.to);
							dist[e.to] = dist[cur] + e.cost + h[cur] - h[e.from];
							prev[e.to] = e;
							set.add(e.to);
						}
					}
				}
				if (dist[n - 1] == INF) {
					return;
				}
				int cur = n - 1;
				while (cur != 0) {
					Edge pr = prev[cur];
					pr.flow++;
					pr.rev.flow--;
					cur = pr.from;
				}
				for (int i = 0; i < n; i++) {
					if (dist[i] != INF) {
						h[i] += dist[i];
					}
				}
			}
		}

		private long[] getPotentials() {
			int n = graph.length;
			long[] dist = new long[n];
			int[] queue = new int[n * n];
			boolean[] inQueue = new boolean[n];
			long INF = Long.MAX_VALUE;
			Arrays.fill(dist, INF);
			dist[0] = 0;
			int head = 0, tail = 0;
			queue[tail++] = 0;
			inQueue[0] = true;
			while (head < tail) {
				int cur = queue[head++];
				inQueue[cur] = false;
				for (Edge e : graph[cur]) {
					if (e.flow < e.cap && dist[e.to] > dist[cur] + e.cost) {
						dist[e.to] = dist[cur] + e.cost;
						if (!inQueue[e.to]) {
							queue[tail++] = e.to;
							inQueue[e.to] = true;
						}
					}
				}
			}
			return dist;
		}

		public Edge addEdgeLowerBound(int from, int to, int min, int max) {
			addEdge(0, to, min, 0);
			addEdge(from, graph.length - 1, min, 0);
			return addEdge(from, to, max - min, 0);
		}
	}

	void solve() {
		int n = in.nextInt(), k = in.nextInt();
		int minS = in.nextInt(), minE = in.nextInt();
		MCMFGraph graph = new MCMFGraph(n + 3);
		int[] s = new int[n], e = new int[n];
		for (int i = 0; i < n; i++) {
			s[i] = -in.nextInt();
		}
		for (int i = 0; i < n; i++) {
			e[i] = -in.nextInt();
		}
		MCMFGraph.Edge[] eat = new MCMFGraph.Edge[n];
		MCMFGraph.Edge[] sleep = new MCMFGraph.Edge[n];
		for (int i = 0; i < n; i++) {
			if (i >= k - 1) {
				sleep[i] = graph.addEdgeLowerBound(i + 1, i + 2, minS, k - minE);
			} else {
				sleep[i] = graph.addEdgeLowerBound(i + 1, i + 2, 0, k);
			}
			eat[i] = graph.addEdge(i + 1, Math.min(n, i + k) + 1, 1, e[i] - s[i]);
		}
		graph.addEdgeLowerBound(n + 1, 1, k, k);
		graph.minCostFlow();
		long result = 0;
		for (int i = 0; i < n; i++) {
			result -= (eat[i].flow == 1 ? e[i] : s[i]);
		}
		out.println(result);
		for (int i = 0; i < n; i++) {
			out.print(eat[i].flow == 1 ? "E" : "S");
		}
		out.println();
	}

	FastScanner in;
	PrintWriter out;

	void run() {
		in = new FastScanner("delight.in");
		try {
			out = new PrintWriter("delight.out");
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		solve();
		out.close();
	}

	class FastScanner {
		BufferedReader br;
		StringTokenizer st;

		public FastScanner() {
			br = new BufferedReader(new InputStreamReader(System.in));
		}

		public FastScanner(String s) {
			try {
				br = new BufferedReader(new FileReader(s));
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		public String nextToken() {
			while (st == null || !st.hasMoreTokens()) {
				try {
					st = new StringTokenizer(br.readLine());
				} catch (IOException e) {
				}
			}
			return st.nextToken();
		}

		public int nextInt() {
			return Integer.parseInt(nextToken());
		}

		public long nextLong() {
			return Long.parseLong(nextToken());
		}

		public double nextDouble() {
			return Double.parseDouble(nextToken());
		}
	}

	public static void main(String[] args) {
		new delight_av_tl2().run();
	}
}
