#include <vector>
#include <algorithm>
#include <cassert>
#include <cstdio>

using namespace std;

const int inf = (int)1e9;

typedef long long ll;

class MinCostCirculation {
    struct Edge {
      int to, capacity, cost, flow;
    };
    vector<Edge> es;
    int n;
    ll curRes;
  public:

    MinCostCirculation(int n) : es(), n(n), curRes(0) {}

    int add_edge(int from, int to, int capacity, int cost) {
      Edge e;
      e.to = to;
      e.capacity = capacity;
      e.cost = cost;
      e.flow = 0;
      es.push_back(e);
      e.to = from;
      e.capacity = 0;
      e.cost = -cost;
      e.flow = 0;
      es.push_back(e);
      return (int)(es.size()) - 2;
    }

    void push_flow(int eid, int minv) {
      assert(es[eid].capacity >= minv);
      es[eid].capacity -= minv;
      es[eid ^ 1].capacity += minv;
      es[eid].flow += minv;
      es[eid ^ 1].flow -= minv;
      curRes += es[eid].cost * 1LL * minv;
    }

    void check() {
      vector<int> v(n);
      for (int i = 0; i < (int)es.size(); i++) {
        if (es[i].flow > 0) {
          v[es[i].to] += es[i].flow;
          v[es[i ^ 1].to] -= es[i].flow;
        }
      }
      for (int i = 0; i < n; i++) {
        fprintf(stderr, "%d ", v[i]);
      }
      fprintf(stderr, "\n");
      for (int i = 0; i < n; i++) {
        assert(v[i] == 0);
      }
    }

    ll solve() {
      while (true) {
        vector<ll> d(n);
        vector<int> fre(n, -1);
        int base = -1;
        for (int step = 0; step < n; step++) {
          for (int i = 0; i < (int)(es.size()); i++) {
            if (es[i].capacity > 0) {
              int b = es[i].to;
              int a = es[i ^ 1].to;
              if (d[b] <= d[a] + es[i].cost) {
                continue;
              }
              d[b] = d[a] + es[i].cost;
              fre[b] = i;
              if (step == n - 1) {
                base = b;
              }
            }
          }
        }
        if (base < 0) {
          break;
        }
        vector<int> seq;
        vector<bool> was(n, false);
        for (int x = base;; x = es[fre[x] ^ 1].to) {
          if (!was[x]) {
            seq.push_back(x);
            was[x] = true;
          } else {
            seq.erase(seq.begin(), find(seq.begin(), seq.end(), x));
            break;
          }
        }

        int minv = inf;
        for (int i = 0; i < (int)(seq.size()); i++) {
          int v = seq[i];
          int eid = fre[v];
          minv = min(minv, es[eid].capacity);
        }

        for (int i = 0; i < (int)(seq.size()); i++) {
          int v = seq[i];
          int eid = fre[v];
          push_flow(eid, minv);
        }
      }

      return curRes;
    }

    int get_capacity(int id) {
      return es[id].capacity;
    }
};

int main() {
  assert(freopen("delight.in", "r", stdin));
  assert(freopen("delight.out", "w", stdout));

  int n, k;
  int min_s, min_e;
  assert(scanf("%d%d%d%d", &n, &k, &min_s, &min_e) == 4);
  int max_e = k - min_s;
  vector<int> e(n), s(n);
  for (int &x : s) {
    assert(scanf("%d", &x) == 1);
  }
  for (int &x : e) {
    assert(scanf("%d", &x) == 1);
  }

  MinCostCirculation solver(n + 2);
  vector<int> up_edge(n);

  for (int i = 0; i < n; i++) {
    solver.add_edge(i, i + 1, max_e - min_e, 0);
  }
  ll ans = 0;
  for (int i = 0; i < n; i++) {
    ans += s[i];
    up_edge[i] = solver.add_edge(i, min(i + k, n), 1, s[i] - e[i]);
    if (i % k < max_e) {
      solver.push_flow(up_edge[i], 1);
    }
  }
//  int ts_edge = solver.add_edge(n, n + 1, max_e, 0);
//  solver.push_flow(ts_edge, max_e);

  for (int i = 0; i < k; i++) {
    int id = solver.add_edge(n + 1, i, 1, 0);
    if (i < max_e) {
      solver.push_flow(id, 1);
    }
  }
//  solver.check();
  ans -= solver.solve();
//  assert(solver.get_capacity(ts_edge) == 0);
  printf("%lld\n", ans);
  for (int i = 0; i < n; i++) {
    if (solver.get_capacity(up_edge[i])) {
      printf("S");
    } else {
      printf("E");
    }
  }
  printf("\n");
  return 0;
}
