#include <vector>
#include <algorithm>
#include <cassert>
#include <cstdio>
#include <queue>

using namespace std;

const int inf = (int)1e9;

typedef long long ll;

class MinCostFlow {
    struct Edge {
      int to, capacity, cost, next;
    };
    vector<Edge> es;
    vector<int> first;
    int n;
    ll curRes;
  public:

    MinCostFlow(int n) : es(), n(n), curRes(0), first(n, -1) {}

    int add_edge(int from, int to, int capacity, int cost) {
      Edge e;
      e.to = to;
      e.capacity = capacity;
      e.cost = cost;
      e.next = first[from];
      first[from] = (int)es.size();
      es.push_back(e);
      e.to = from;
      e.capacity = 0;
      e.cost = -cost;
      e.next = first[to];
      first[to] = (int)es.size();
      es.push_back(e);
      return (int)(es.size()) - 2;
    }

    void push_flow(int eid, int minv) {
      assert(es[eid].capacity >= minv);
      es[eid].capacity -= minv;
      es[eid ^ 1].capacity += minv;
      curRes += es[eid].cost * 1LL * minv;
    }

    ll solve(int s, int t, int need_flow = inf) {
      while (need_flow > 0) {
        vector<ll> d(n, 1LL << 60);
        d[s] = 0;
        vector<int> fre(n, -1);
        deque<int> q;
        vector<int> inq(n, false);
        q.push_back(s);
        inq[s] = true;
        while (!q.empty()) {
          int a = q.front();
          q.pop_front();
          inq[a] = false;
          for (int i = first[a]; i != -1; i = es[i].next) {
            if (es[i].capacity > 0) {
              int b = es[i].to;
              if (d[b] <= d[a] + es[i].cost) {
                continue;
              }
              if (!inq[b]) {
                q.push_back(b);
                inq[b] = true;
              }
              d[b] = d[a] + es[i].cost;
              fre[b] = i;
            }
          }
        }

        vector<int> seq;
        for (int x = t; x != s; x = es[fre[x] ^ 1].to) {
          seq.push_back(x);
        }

        int minv = need_flow;
        for (int i = 0; i < (int)(seq.size()); i++) {
          int v = seq[i];
          int eid = fre[v];
          minv = min(minv, es[eid].capacity);
        }


        for (int i = 0; i < (int)(seq.size()); i++) {
          int v = seq[i];
          int eid = fre[v];
          push_flow(eid, minv);
        }
        need_flow -= minv;
      }
      return curRes;
    }

    int get_capacity(int id) {
      return es[id].capacity;
    }
};

int main() {
  assert(freopen("delight.in", "r", stdin));
  assert(freopen("delight.out", "w", stdout));

  int n, k;
  int min_s, min_e;
  assert(scanf("%d%d%d%d", &n, &k, &min_s, &min_e) == 4);
  int max_e = k - min_s;
  vector<int> e(n), s(n);
  for (int &x : s) {
    assert(scanf("%d", &x) == 1);
  }
  for (int &x : e) {
    assert(scanf("%d", &x) == 1);
  }

  MinCostFlow solver(n + 2);
  vector<int> up_edge(n);

  for (int i = 0; i < n; i++) {
    solver.add_edge(i, i + 1, max_e - min_e, 0);
  }
  ll ans = 0;
  for (int i = 0; i < n; i++) {
    ans += s[i];
    up_edge[i] = solver.add_edge(i, min(i + k, n), 1, s[i] - e[i]);
  }

  for (int i = 0; i < k; i++) {
    int id = solver.add_edge(n + 1, i, 1, 0);
  }
  ans -= solver.solve(n + 1, n, max_e);

  printf("%lld\n", ans);
  for (int i = 0; i < n; i++) {
    if (solver.get_capacity(up_edge[i])) {
      printf("S");
    } else {
      printf("E");
    }
  }
  printf("\n");
  return 0;
}
