import java.io.*;
import java.util.*;

public class delight_tourist {

	public static class MCMF {
		static int n;
		static int[] from;
		static int[] to;
		static int[] cap;
		static int[] flow;
		static int[] cost;
		static int[] rev;
		static int[] prev;
		static int[] last;
		static int m;

		public static void init(int N, int size) {
			n = N;
			from = new int[size];
			to = new int[size];
			cap = new int[size];
			flow = new int[size];
			cost = new int[size];
			rev = new int[size];
			prev = new int[size];
			last = new int[size];
			m = 0;
			for (int i = 0; i < n; i++) {
				last[i] = -1;
			}
		}

		public static void addSimple(int _from, int _to, int _cap, int _cost, int _rev) {
			from[m] = _from;
			to[m] = _to;
			cap[m] = _cap;
			flow[m] = 0;
			cost[m] = _cost;
			rev[m] = _rev;
			prev[m] = last[from[m]];
			last[from[m]] = m;
			m++;
		}

		public static void add(int from, int to, int cap, int cost) {
			addSimple(from, to, cap, cost, m + 1);
			addSimple(to, from, 0, -cost, m - 1);
		}

		public static long findPath(int start, int finish) {
			long[] distance = new long[n];
			int[] prevEdge = new int[n];
			boolean[] inQueue = new boolean[n];
			for (int i = 0; i < n; i++) {
				distance[i] = Long.MAX_VALUE;
				prevEdge[i] = -1;
				inQueue[i] = false;
			}
			Queue<Integer> queue = new LinkedList<>();
			queue.add(start);
			distance[start] = 0;
			inQueue[start] = true;
			while (!queue.isEmpty()) {
				int ver = queue.remove();
				inQueue[ver] = false;
				int j = last[ver];
				while (j >= 0) {
					if (cap[j] > flow[j]) {
						long newDistance = distance[ver] + cost[j];
						if (newDistance < distance[to[j]]) {
							distance[to[j]] = newDistance;
							prevEdge[to[j]] = j;
							if (!inQueue[to[j]]) {
								queue.add(to[j]);
								inQueue[to[j]] = true;
							}
						}
					}
					j = prev[j];
				}
			}
			if (distance[finish] != Long.MAX_VALUE) {
				int ver = finish;
				while (ver != start) {
					int edge = prevEdge[ver];
					flow[edge]++;
					if (rev[edge] != -1) {
						flow[rev[edge]]--;
					}
					ver = from[edge];
				}
			}
			return distance[finish];
		}

		public static long fixedFlowMinCost(int start, int finish, int flowSize) {
			long result = 0;
			while (flowSize-- > 0) {
				long value = findPath(start, finish);
				if (value == Long.MAX_VALUE) {
					return -1;
				}
				result += value;
			}
			return result;
		}
	}

	public static void main(String[] args) throws IOException {
		Scanner in = new Scanner(new File("delight.in"));
		PrintWriter out = new PrintWriter("delight.out");
		int n = in.nextInt();
		int period = in.nextInt();
		int minSleep = in.nextInt();
		int minEat = in.nextInt();
		int[] S = new int[n];
		for (int i = 0; i < n; i++) {
			S[i] = in.nextInt();
		}
		int[] E = new int[n];
		for (int i = 0; i < n; i++) {
			E[i] = in.nextInt();
		}
		int maxSleep = period - minEat;
		int maxEat = period - minSleep;
		MCMF.init(n + 2, 10 * n);
		long result = 0;
		for (int i = 0; i < n; i++) {
			result += S[i];
			MCMF.add(i, Math.min(i + period, n), 1, S[i] - E[i]);
		}
		for (int i = 0; i < period; i++) {
			MCMF.add(n + 1, i, period, 0);
		}
		for (int i = 0; i < n; i++) {
			MCMF.add(i, i + 1, maxSleep + maxEat - period, 0);
		}
		result -= MCMF.fixedFlowMinCost(n + 1, n, maxEat); 
		StringBuilder actions = new StringBuilder();
		for (int i = 0; i < n; i++) {
			if (MCMF.flow[2 * i] == 0) {
				actions.append("S");
			} else {
				actions.append("E");
			}
		}
		out.println(result);
		out.println(actions.toString());
		in.close();
		out.close();
	}
}
