import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * Created by Aksenov239 on 19.11.2016.
 */
public class Tests {
    final int SMALL_N = 10;
    final int SMALL_Q = 10;
    final int SMALL_T = 50;
    final int SMALL_X = 50;
    final int SMALL_B = SMALL_N * SMALL_X;

    final int MAX_N = 100_000;
    final int MAX_Q = 100_000;
    final int MAX_T = 1_000_000_000;
    final int MAX_X = 10_000;
    final int MAX_B = MAX_N * MAX_X;

    int tests = 1;

    public void writeTest(Operation[] operations, int[] queries) {
        try {
            PrintWriter out = new PrintWriter(String.format("tests/%02d", tests++));

            System.err.println("Writing test " + (tests - 1));

            out.println(operations.length + " " + queries.length);
            for (Operation operation : operations) {
                out.println((operation.take ? "-" : "+") + " " + operation.t + " " + operation.x);
            }
            for (int i = 0; i < queries.length; i++) {
                out.print((i == 0 ? "" : " ") + queries[i]);
            }
            out.println();
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    Random rnd = new Random(239);

    public class Operation {
        boolean take;
        int t;
        int x;

        public Operation(boolean take, int x) {
            this.take = take;
            this.x = x;
        }

        public Operation(boolean take, int t, int x) {
            this(take, x);
            this.t = t;
        }
    }

    public int[] randomChoose(int MIN_X, int MAX_X, int k) {
        if (k == 0 || MIN_X >= MAX_X) {
            return new int[0];
        }
        if (MAX_X - MIN_X < 10 * k) {
            ArrayList<Integer> toChoose = new ArrayList<>();
            for (int i = MIN_X; i < MAX_X; i++) {
                toChoose.add(i);
            }
            Collections.shuffle(toChoose, rnd);
            int[] result = new int[Math.min(k, MAX_X - MIN_X)];
            for (int i = 0; i < result.length; i++) {
                result[i] = toChoose.get(i);
            }
            return result;
        } else {
            HashSet<Integer> taken = new HashSet<>();
            int[] results = new int[k];
            for (int i = 0; i < k; i++) {
                int nr;
                do {
                    nr = rnd.nextInt(MAX_X - MIN_X) + MIN_X;
                } while (taken.contains(nr));
                taken.add(nr);
                results[i] = nr;
            }
            return results;
        }
    }

    public void fillRandomT(Operation[] operations, int MAX_T) {
        int[] t = randomChoose(1, MAX_T + 1, operations.length);
        Arrays.sort(t);
        for (int i = 0; i < t.length; i++) {
            operations[i].t = t[i];
        }
    }

    public int[] genQueries(Operation[] operations, int m, int MAX_B) {
        int[] y = new int[operations.length + 1];
        HashSet<Integer> taken = new HashSet<>();
        int currentLevel = 0;
        int maxLevel = 0;
        int len = 0;
        y[len++] = 0;
        taken.add(0);
        for (int i = 0; i < operations.length; i++) {
            currentLevel += operations[i].take ? operations[i].x : -operations[i].x;
            if (currentLevel > 0) {
                if (taken.contains(currentLevel))
                    continue;
                taken.add(currentLevel);
                y[len++] = currentLevel;
                maxLevel = Math.max(maxLevel, currentLevel);
            }
        }
        y = Arrays.copyOf(y, len);
        Arrays.sort(y);

        int samples = 10 * m / y.length;

        int[] possibleQueries = new int[samples * y.length + 1 + samples];

        len = 0;
        for (int i = 0; i < y.length - 1; i++) {
            if (currentLevel >= y[i + 1])
                continue;
            int[] s = randomChoose(Math.max(y[i], currentLevel), y[i + 1], samples - 1);
            possibleQueries[len++] = y[i];
            for (int j = 0; j < s.length; j++) {
                possibleQueries[len++] = s[j];
            }
        }
        possibleQueries[len++] = y[y.length - 1];
        int[] s = randomChoose(0, currentLevel, (possibleQueries.length - len) / 2);
        for (int j = 0; j < s.length; j++) {
            possibleQueries[len++] = s[j];
        }
        s = randomChoose(y[y.length - 1] + 1, MAX_B + 1, (possibleQueries.length - len));
        for (int j = 0; j < s.length; j++) {
            possibleQueries[len++] = s[j];
        }

        int[] forced = new int[]{0, MAX_B, y[y.length - 1], y[y.length - 1] - 1, currentLevel - 1, currentLevel};
        int[] chosen = randomChoose(0, len, m);
        taken.clear();
        ArrayList<Integer> queries = new ArrayList<>();
        for (int i = 0; i < forced.length; i++) {
            if (forced[i] > MAX_B || forced[i] < 0 || taken.contains(forced[i])) {
                continue;
            }
            queries.add(forced[i]);
            taken.add(forced[i]);
        }

        for (int i = 0; i < chosen.length; i++) {
            if (queries.size() == m) {
                break;
            }
            if (taken.contains(chosen[i])) {
                continue;
            }
            queries.add(chosen[i]);
        }
        Collections.shuffle(queries, rnd);
        return queries.stream().mapToInt(Integer::intValue).toArray();
    }

    public void writeTest(Operation[] operations, int m, int MAX_T, int MAX_B) {
        fillRandomT(operations, MAX_T);
        writeTest(operations, genQueries(operations, m, MAX_B));
    }

    public void genOneSideRandomLadder(boolean maxTest, boolean up, int n, int m, int MAX_X, int MAX_T, int MAX_B) {
        Operation[] operations = new Operation[n];
        for (int i = 0; i < n; i++) {
            operations[i] = new Operation(up, maxTest ? MAX_X : rnd.nextInt(MAX_X) + 1);
        }
        writeTest(operations, m, MAX_T, MAX_B);
    }

    public void genSymmetricLadder(boolean maxTest, boolean up, int n, int m, int MAX_X, int MAX_T, int MAX_B) {
        Operation[] operations = new Operation[n];
        for (int i = 0; i < (operations.length + 1) / 2; i++) {
            operations[i] = new Operation(up, maxTest ? MAX_X : rnd.nextInt(MAX_X) + 1);
        }
        for (int i = (operations.length + 1) / 2; i < operations.length; i++) {
            operations[i] = new Operation(!up, operations[operations.length - 1 - i].x);
        }
        writeTest(operations, m, MAX_T, MAX_B);
    }

    public void genPseudoRandomLadder(boolean one_half, boolean up, int n, int m, int MAX_X, int MAX_T, int MAX_B, int max_delta) {
        Operation[] operations = new Operation[n];
        int len = 0;

        int currentLevel = 0;
        boolean direction = up;
        while (len < n) {
            int delta = 0;
            if (one_half) {
                if (direction == up) {
                    delta = rnd.nextInt(max_delta) + 1;
                } else {
                    delta = rnd.nextDouble() < 0.1 ? Math.abs(currentLevel) : rnd.nextInt(Math.abs(currentLevel)) + 1;
                }
            } else {
                int sign = direction == up ? 1 : -1;
                if (currentLevel * sign >= 0) {
                    delta = rnd.nextInt(max_delta) + 1;
                } else {
                    delta = rnd.nextBoolean() ? rnd.nextInt(Math.abs(currentLevel)) + 1 : (rnd.nextInt(max_delta + 1) + Math.abs(currentLevel));
                }
            }

            currentLevel += direction == up ? delta : -delta;

            while (len < n && delta > 0) {
                int x = Math.min(rnd.nextInt(MAX_X) + 1, delta);
                operations[len++] = new Operation(direction, x);
                delta -= x;
            }

            direction = !direction;
        }
        writeTest(operations, m, MAX_T, MAX_B);
    }

    public void genRandomLadder(int n, int m, int MAX_X, int MAX_T, int MAX_B) {
        Operation[] operations = new Operation[n];
        for (int i = 0; i < operations.length; i++) {
            operations[i] = new Operation(rnd.nextBoolean(), rnd.nextInt(MAX_X) + 1);
        }
        writeTest(operations, m, MAX_T, MAX_B);
    }

    public void genSample() {
        writeTest(
                new Operation[]{
                        new Operation(true, 1, 1),
                        new Operation(true, 2, 2),
                        new Operation(false, 4, 1),
                        new Operation(true, 6, 1),
                        new Operation(false, 7, 2)},
                new int[]{0, 3, 1, 2});
    }

    public void gen() {
        genSample();

        genOneSideRandomLadder(false, true, SMALL_N, SMALL_Q, SMALL_X, SMALL_T, SMALL_B);
        genOneSideRandomLadder(false, false, SMALL_N, SMALL_Q, SMALL_X, SMALL_T, SMALL_B);
        genSymmetricLadder(false, true, SMALL_N, SMALL_Q, SMALL_X, SMALL_T, SMALL_B);
        genSymmetricLadder(false, false, SMALL_N, SMALL_Q, SMALL_X, SMALL_T, SMALL_B);
        for (int mask = 0; mask < 4; mask++) {
            genPseudoRandomLadder((mask & 1) == 0, (mask & 2) == 0, SMALL_N, SMALL_Q, SMALL_X, SMALL_T, SMALL_B, (int) Math.sqrt(SMALL_X * SMALL_N));
        }
        genRandomLadder(SMALL_N, SMALL_Q, SMALL_X, SMALL_T, SMALL_B);

        for (int i = 0; i < 2; i++) {
            genOneSideRandomLadder(false, true, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B);
            genOneSideRandomLadder(false, false, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B);
        }
        genOneSideRandomLadder(true, true, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B);
        genOneSideRandomLadder(true, false, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B);

        for (int i = 0; i < 2; i++) {
            genSymmetricLadder(false, true, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B);
            genSymmetricLadder(false, false, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B);
        }
        genSymmetricLadder(true, true, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B);
        genSymmetricLadder(true, false, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B);

        for (int i = 0; i < 2; i++) {
            genSymmetricLadder(false, true, MAX_N - 1, MAX_Q, MAX_X, MAX_T, MAX_B);
            genSymmetricLadder(false, false, MAX_N - 1, MAX_Q, MAX_X, MAX_T, MAX_B);
        }

        for (int i = 0; i < 4; i++) {
            for (int mask = 0; mask < 4; mask++) {
                genPseudoRandomLadder((mask & 1) == 0, (mask & 2) == 0, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B, rnd.nextInt(MAX_X) * rnd.nextInt(MAX_N) + 1);
            }
        }
        for (int i = 0; i < 2; i++) {
            for (int mask = 0; mask < 4; mask++) {
                genPseudoRandomLadder((mask & 1) == 0, (mask & 2) == 0, MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B, (int) Math.sqrt(MAX_X * MAX_N));
            }
        }

        for (int i = 0; i < 4; i++) {
            genRandomLadder(MAX_N, MAX_Q, MAX_X, MAX_T, MAX_B);
        }
    }

    public static void main(String[] args) {
        new Tests().gen();
    }
}
