#ifdef DBG1
  #define _GLIBCXX_DEBUG
#endif

#include <cstdio>
#include <cassert>
#include <cstring>
#include <string>
#include <vector>
#include <algorithm>
#include <map>
#include <set>

#ifdef DBG1
    #define eprintf(...) fprintf(stderr, __VA_ARGS__),fflush(stderr)
#else
    #define eprintf(...)
#endif

using namespace std;

typedef long long ll;
typedef unsigned long long ull;
typedef pair <int, int> pii;

#define TASKNAME "expect"

const int INF = int(1e9) + 100;

struct Arc {
  int ts, tf;
  int cnt;
  int pot;
  vector <int> next;

  Arc(int ts, int tf, int cnt) : ts(ts), tf(tf), cnt(cnt), pot(0) {}

  bool operator < (const Arc & other) const {
    if (ts != other.ts) {
      return ts < other.ts;
    }
    return tf > other.tf;
  }
};

int main() {
  freopen(TASKNAME ".in", "r", stdin);
  freopen(TASKNAME ".out", "w", stdout);

  int n, q;
  scanf("%d%d", &n, &q);
  int pluses = 0;
  vector <pii> minuses;
  vector <Arc> arcs;
  arcs.push_back(Arc(0, int(1e9) + 1, 0));
  for (int i = 0; i < n; ++i) {
    int t, k;
    char ch;
    scanf(" %c%d%d", &ch, &t, &k);
    if (ch == '+') {
      while (!minuses.empty() && k > 0) {
        int x = min(minuses.back().second, k);
        arcs.push_back(Arc(minuses.back().first, t, x));
        k -= x;
        minuses.back().second -= x;
        if (minuses.back().second == 0) {
          minuses.pop_back();
        }
      }
      pluses += k;
    } else if (ch == '-') {
      int x = min(pluses, k);
      pluses -= x;
      k -= x;
      if (k) {
        minuses.push_back(pii(t, k));
      }
    } else {
      assert(0);
    }
  }

  while (!minuses.empty()) {
    arcs.push_back(Arc(minuses.back().first, INF, minuses.back().second));
    minuses.pop_back();
  }

  eprintf("read input\n");

  sort(arcs.begin(), arcs.end());
  vector <int> arc_st;
  arc_st.push_back(0);
  for (int i = 1; i < (int)arcs.size(); ++i) {
    while (1) {
      assert(!arc_st.empty());
      if (arcs[arc_st.back()].tf < arcs[i].ts) {
        arc_st.pop_back();
      } else {
        break;
      }
    }
    Arc & par = arcs[arc_st.back()];
    arcs[i].pot = par.pot + arcs[i].cnt;
    eprintf("ts %d tf %d cnt %d pot %d par %d\n", arcs[i].ts, arcs[i].tf, arcs[i].cnt, arcs[i].pot, arc_st.back());
    par.next.push_back(i);
    arc_st.push_back(i);
  }

  eprintf("build arc tree\n");

  vector <ll> ans(q, -2);
  vector <pii> b(q);
  for (int i = 0; i < q; ++i) {
    scanf("%d", &b[i].first);
    b[i].second = i;
  }
  sort(b.begin(), b.end());

  int totInf = 0;
  int totSum = 0;
  set <pii> s; 
  ll curAns = 0;
  int curPot = 0;
  for (const Arc & arc : arcs) {
    if (arc.tf != INF) {
      curAns += ll(arc.cnt) * (arc.tf - arc.ts);
    }
  }
  for (int i : arcs[0].next) {
    s.insert(pii(arcs[i].pot, i));
    if (arcs[i].tf == INF) {
      totInf += 1;
    } else {
      totSum += arcs[i].tf - arcs[i].ts;
    }
  }
  int ptr = 0;
  while (ptr < int(b.size())) {
    if (b[ptr].first == curPot || s.empty()) {
      eprintf("ans[%d] %d = %lld\n", b[ptr].second, b[ptr].first, curAns);
      ans[b[ptr].second] = totInf > 0 ? -1 : curAns;
      ptr++;
    } else {
      int x = min(s.begin()->first, b[ptr].first);
      eprintf("curPot %d ptr %d b[ptr] %d x %d\n", curPot, ptr, b[ptr].first, x);
      assert(x > curPot);
      curAns -= ll(x - curPot) * totSum;
      while (!s.empty() && s.begin()->first == x) {
        int i = s.begin()->second;
        s.erase(s.begin());
        if (arcs[i].tf == INF) {
          totInf -= 1;
        } else {
          totSum -= arcs[i].tf - arcs[i].ts;
        }
        for (int j : arcs[i].next) {
          s.insert(pii(arcs[j].pot, j));
          if (arcs[j].tf == INF) {
            totInf += 1;
          } else {
            totSum += arcs[j].tf - arcs[j].ts;
          }
        }
      }
      curPot = x;
    }
  }

  for (int i = 0; i < q; ++i) {
    assert(ans[i] >= -1);
    if (ans[i] == -1) {
      printf("INFINITY\n");
    } else {
      printf("%lld\n", ans[i]);
    }
  }

  return 0;
}
