import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Random;

/**
 * Created by Niyaz Nigmatullin on 18.11.16.
 */
public class Tests {

    static File testsDirectory;
    static int currentTest;
    static Random rng = new Random(23901750216L);

    static void printTest(String s) {
        ++currentTest;
        try {
            PrintWriter out = new PrintWriter(new File(testsDirectory, String.format("%02d", currentTest)));
            out.println(s.replace('0', 'C').replace('1', 'W'));
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    static String randomTest(int length, int c0, int c1) {
        StringBuilder sb = new StringBuilder(length);
        for (int i = 0; i < length; i++) {
            sb.append(rng.nextInt(c0 + c1) < c0 ? 0 : 1);
        }
        return sb.toString();
    }

    static String flip(String s) {
        char[] c = s.toCharArray();
        for (int i = 0; i < c.length; i++) {
            c[i] = c[i] == '0' ? '1' : '0';
        }
        return new String(c);
    }

    static String flipSome(String s, int k) {
        char[] c = s.toCharArray();
        for (int it = 0; it < k; it++) {
            int i = rng.nextInt(c.length);
            c[i] = c[i] == '0' ? '1' : '0';
        }
        return new String(c);
    }

    static String zebra(int length, int c0, int c1) {
        StringBuilder sb = new StringBuilder(2 * length);
        for (int i = 0; sb.length() < length; i++) {
            int count = (i & 1) == 0 ? c0 : c1;
            char what = (char) ((i & 1) + '0');
            for (int j = 0; j < count; j++) {
                sb.append(what);
            }
        }
        sb.setLength(length);
        return sb.toString();
    }

    static final int MAXN = 1000000;

    static void generateTests() {
        // samples
        printTest("0100");
        printTest("1101001");
        // main testset
        printTest(randomTest(1, 0, 1));
        printTest(randomTest(1, 1, 0));
        for (int length = 2; length <= 10; length++) {
            printTest(randomTest(length, 1, 1));
        }
        for (int length : new int[]{10, 15, 20, 30, 37, 42, 47}) {
            printTest(randomTest(length, rng.nextInt(3) + 1, rng.nextInt(3) + 1));
            if (rng.nextBoolean()) {
                String s = flipSome(randomTest(length, 0, 1), rng.nextInt(10) + 1);
                if (rng.nextBoolean()) {
                    s = flip(s);
                }
                printTest(s);
            }
        }
        printTest(zebra(MAXN, 1, 1));
        printTest(zebra(MAXN, 1, 3));
        printTest(flipSome(randomTest(MAXN, 1, 0), 1));
        printTest(zebra(MAXN, MAXN - 1, 1));
        printTest(flip(zebra(MAXN, MAXN - 1, 1)));
        printTest(flip(zebra(MAXN, 1, 1)));
        for (int length : new int[]{50, 117, 1000, 5000,
                                    50000, 500000}) {
            printTest(randomTest(length, 1, 1));
            if (rng.nextBoolean()) {
                printTest(randomTest(length, 1, rng.nextInt(10) + 3));
            } else {
                printTest(randomTest(length, rng.nextInt(10) + 3, 1));
            }
            if (rng.nextBoolean()) {
                printTest(flipSome(randomTest(length, 0, 1), rng.nextInt(10) + 1));
            } else {
                printTest(flipSome(randomTest(length, 1, 0), rng.nextInt(10) + 1));
            }
        }
        for (int length : new int[]{765432, 7, 15, 12345}) {
            printTest(zebra(length, 1, 1));
            printTest(zebra(length, 1, 2));
        }
    }

    public static void main(String[] args) {
        testsDirectory = new File("tests");
        if (!testsDirectory.exists()) {
            if (!testsDirectory.mkdirs()) {
                System.err.println("couldn't create 'tests' folder");
                System.exit(1);
            }
        }
        currentTest = 0;
        generateTests();
    }
}
