#include <cstdio>
#include <vector>
#include <string>
#include <queue>
#include <cassert>

using namespace std;

vector<vector<int>> gt;
vector<vector<int>> g;

vector<bool> bfs_win() {
  int n = gt.size();
  vector<bool> res(n, false);

  vector<int> deg(n);
  vector<int> len(n, -1);
  queue<int> q;
  for (int v = 0; v < n; v++) {
    for (int u : gt[v]) {
      deg[u]++;
    }
  }

  for (int v = 0; v < n; v++) {
    if (deg[v] == 0) {
      q.push(v);
      len[v] = 0;
    }
  }

  while (!q.empty()) {
    int v = q.front(); q.pop();
    if (len[v] % 2 == 0) {
      res[v] = v % 2 == 0;
      for (int u : gt[v]) {
        if (len[u] == -1) {
          len[u] = len[v] + 1;
          q.push(u);
        }
      }
    } else {
      res[v] = v % 2 == 1;
      for (int u : gt[v]) {
        deg[u]--;
        if (len[u] == -1 && deg[u] == 0) {
          len[u] = len[v] + 1;
          q.push(u);
        }
      }
    }
  }

  return res;
}

vector<bool> bfs_draw() {
  int n = gt.size();
  vector<bool> res(n, true);

  vector<int> deg(n);
  queue<int> q;
  for (int v = 0; v < n; v++) {
    for (int u : gt[v]) {
      deg[u]++;
    }
  }

  for (int v = 0; v < n; v++) {
    if (deg[v] == 0) {
      q.push(v);
    }
  }

  while (!q.empty()) {
    int v = q.front(); q.pop();
    res[v] = false; 
    if (v % 2 == 0) {
      for (int u : gt[v]) {
        if (deg[u] > 0) {
          q.push(u);
          deg[u] = -1;
        }
      }
    } else {
      for (int u : gt[v]) {
        deg[u]--;
        if (deg[u] == 0) {
          q.push(u);
        }
      }
    }
  }

  return res;
}

void checkConsistency(string ans1, string ans2) {
  for (int i = 0; i < (int)g.size(); i++) {
    bool haveL, haveD, haveW;
    haveL = haveD = haveW = false;
    for (int u : g[i]) {
      if (ans2[u] == 'L') haveL = true;
      if (ans2[u] == 'D') haveD = true;
      if (ans2[u] == 'W') haveW = true;
    }
    if (ans1[i] == 'D') {
      assert(haveD);
    } else if (ans1[i] == 'W') {
      assert(!haveD && haveL);
    } else {
      assert(!haveD && !haveL);
    }
  }
  for (int i = 0; i < (int)g.size(); i++) {
    bool haveL, haveD, haveW;
    haveL = haveD = haveW = false;
    for (int u : g[i]) {
      if (ans1[u] == 'L') haveL = true;
      if (ans1[u] == 'D') haveD = true;
      if (ans1[u] == 'W') haveW = true;
    }
    if (ans2[i] == 'W') {
      assert(haveL);
    } else if (ans2[i] == 'L') {
      assert(!haveL && (haveW || g[i].size() == 0));
    } else {
      assert(!haveL && !haveW);
    }
  }
}

int main() {
  assert(freopen("game.in", "r", stdin));
  assert(freopen("game.out", "w", stdout));

  int n, m;
  assert(scanf("%d%d", &n, &m) == 2);
  gt.resize(2 * n);
  g.resize(n);
  for (int i = 0; i < m; i++) {
    int a, b;
    assert(scanf("%d%d", &a, &b) == 2);
    --a, --b;
    gt[2 * b].push_back(2 * a + 1);
    gt[2 * b + 1].push_back(2 * a);
    g[a].push_back(b);
  }

  vector<bool> res_win = bfs_win();
  vector<bool> res_draw = bfs_draw();

  string ans1(n, '.');
  string ans2(n, '.');

  for (int i = 0; i < n; i++) {
    if (res_draw[2 * i]) {
      ans1[i] = 'D';
      assert(!res_win[2 * i]);
    } else if (res_win[2 * i]) {
      ans1[i] = 'L';
    } else {
      ans1[i] = 'W';
    }
  }
  
  for (int i = 0; i < n; i++) {
    if (res_win[2 * i + 1]) {
      ans2[i] = 'W';
      assert(!res_draw[2 * i + 1]);
    } else if (res_draw[2 * i + 1]) {
      ans2[i] = 'D';
    } else {
      ans2[i] = 'L';
    }
  }

  checkConsistency(ans1, ans2);

  printf("%s\n%s\n", ans1.c_str(), ans2.c_str());
  return 0;
}
