import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import static ru.ifmo.testlib.Outcome.Type.*;

/**
 * Checker for NEERC'2016 Problem H: Hard Refactoring
 *
 * @author Roman Elizarov
 */
public class Check implements Checker {
    private List<String> nextAns(InStream ouf, Outcome.Type type) {
        String line = ouf.nextLine().trim();
        if (line.equals("true") || line.equals("false"))
            return Collections.singletonList(line);
        List<String> result = new ArrayList<>();
        while (true) {
            boolean endsWithOr = line.endsWith(" ||");
            if (endsWithOr)
                line = line.substring(0, line.length() - 3);
            result.add(line);
            if (ouf.seekEoF()) {
                if (endsWithOr)
                    throw new Outcome(type, "Last line ends with '||'");
                break;
            }
            if (!endsWithOr)
                throw new Outcome(type, "Not-last line missing '||' at the end");
            line = ouf.nextLine().trim();
        }
        // sort for comparison !!!
        Collections.sort(result);
        return result;
    }

    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        List<String> ja = nextAns(ans, FAIL);
        List<String> pa = nextAns(ouf, PE);
        if (pa.size() != ja.size())
            return new Outcome(WA, String.format(Locale.US, "Expected %d lines, found %d line", ja.size(), pa.size()));
        for (int i = 0; i < pa.size(); i++) {
            if (!pa.get(i).equals(ja.get(i)))
                return new Outcome(WA, String.format(Locale.US, "Expected '%s' clause, found '%s' clause", ja.get(i), pa.get(i)));
        }
        return new Outcome(OK, String.format(Locale.US, "%d lines", pa.size()));
    }
}
