import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Scanner;
import java.util.stream.IntStream;
import java.util.stream.Stream;

public class Interact {
    private static final int OK_EXIT_CODE = 0;
    private static final int EARLY_EXIT_CODE = 10;

    static final int MAX_N = 20;
    static final int MAX_M = 20;
    static final int MAX_MOVES = 20000;
    static final int KEY = 324958234;
    static final int MODULO = MAX_N + 1;
    public static final String RIGHT = "right";
    public static final String LEFT = "left";
    public static final String CENTER = "center";
    public static final String TREASURE = "treasure";

    private static int nextInt(Scanner scanner) {
        return (scanner.nextInt() - KEY % MODULO + MODULO) % MODULO;
    }

    private static int run(String inFilename, String outFilename) throws IOException {
        try (Scanner in = new Scanner(new File(inFilename));
             PrintWriter out = new PrintWriter(outFilename)) {
            final int n = nextInt(in);
            final int m = nextInt(in);
            final int[][] cave = Stream.generate(() -> IntStream.generate(() -> nextInt(in)).limit(m).toArray())
                    .limit(n).toArray(int[][]::new);
            final int[] stones = new int[n];
            final String[] sides = Arrays.stream(cave).map(a -> CENTER).toArray(String[]::new);
            boolean[][] visited = new boolean[n][m];
            int passages = n * m;
            int chamber = 0;
            int move = 0;
            try (Scanner pipeIn = new Scanner(System.in);
                     PrintWriter pipeOut = new PrintWriter(System.out);
                     PrintWriter log = new PrintWriter(new File("Interact.log"))) {
                send(pipeOut, log, m + "");
                log.println(passages);
                while (passages > 0) {
                    if (move++ > MAX_MOVES) {
                        return writeResult(out, log, "WA", "Too many moves", EARLY_EXIT_CODE);
                    }
                    send(pipeOut, log, sides[chamber]);
                    try {
                        int stone = pipeIn.nextInt();
                        String side = pipeIn.next();
                        int passage = pipeIn.nextInt();
                        log.println("--- at chamber " + chamber + " with stone at " + stones[chamber]);
                        log.println("<<< " + stone + " " + side + " " + passage);

                        final String check = check(m, stone, side, passage);
                        if (check != null) {
                            return writeResult(out, log, "PE", "Move " + move + ": " + check, EARLY_EXIT_CODE);
                        }

                        passage = (stones[chamber] + passage) % m;
                        stones[chamber] = (stones[chamber] + stone) % m;
                        sides[chamber] = side;
                        if (!visited[chamber][passage]) {
                            visited[chamber][passage] = true;
                            passages--;
                        }
                        chamber = cave[chamber][passage];
                    } catch (Exception e) {
                        return writeResult(out, log, "PE", "Move " + move + ": " + e.getMessage(), EARLY_EXIT_CODE);
                    }
                }
                send(pipeOut, log, TREASURE);
                return writeResult(out, log, "OK", move + " moves", OK_EXIT_CODE);
            }
        }
    }

    private static void send(final PrintWriter pipeOut, final PrintWriter log, final String string) {
        pipeOut.println(string);
        pipeOut.flush();
        log.println(">>> " + string);
        log.flush();
    }

    private static String check(final int m, final int stone, final String side, final int passage) {
        if (stone < 0 || m <= stone) {
            return "Stone out of range: " + stone + " of " + m;
        } else if (!LEFT.equals(side) && !RIGHT.equals(side)) {
            return "Invalid side: " + side;
        } else if (passage < 0 || m <= passage) {
            return "Passage out of range: " + stone + " of " + m;
        }
        return null;
    }

    private static int writeResult(final PrintWriter out, final PrintWriter log, final String outcome, final String comment, int exitCode) {
        log.println(outcome + " " + comment);
        out.println(KEY);
        out.println(outcome);
        out.println(comment);
        return exitCode;
    }

    public static void main(String[] args) throws IOException {
        System.exit(run(
            args.length >= 1 ? args[0] : "indiana.in",
            args.length >= 2 ? args[1] : "indiana.out"
        ));
    }
}
