import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.function.IntBinaryOperator;
import java.util.function.ToIntFunction;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

public class Tests {
    public static final IntBinaryOperator ASC = (a, b) -> b - a;
    public static final IntBinaryOperator DESC = (a, b) -> a - b;
    static int nTests = 0;
    static File root = new File("tests");

    static int MAX_N = Interact.MAX_N;
    static int MAX_M = Interact.MAX_M;
    static Random random = new Random(5712632832434238417L);

    static int e(int v) {
        return (v + Interact.KEY) % Interact.MODULO;
    }

    public static void main(String[] args) throws IOException {
        test(new int[][]{{0, 1}, {0, 0}});
        test(new int[][]{{0, 1}, {2, 2}, {0, 1}});
        test(new int[][]{{2, 1, 1}, {2, 2, 1}, {1, 1, 0}});
        test(new int[][]{{2, 1, 2}, {1, 1, 2}, {0, 2, 2}});
        test(new int[1][2]);
        test(new int[1][MAX_M]);
        for (int s : new int[]{4, MAX_N / 2, MAX_N}) {
            test(shuffle(antiRandom(s, MAX_M)));
            test(shuffle(merge(antiRandom(s / 2, MAX_M), s / 2 - 1, antiRandom(s / 2, MAX_M), s / 2 - 1)));
            test(shuffle(merge(antiRandom(s / 2, MAX_M), s / 2 - 1, tree(s / 2, MAX_M), 0)));

            test(shuffle(cycle(almost(s), 2)));
            test(shuffle(cycle(almost(s), random.nextInt(MAX_M - 3) + 3)));
            test(shuffle(cycle(s, MAX_M)));

            test(shuffle(chain(s, 2)));
            test(shuffle(chain(almost(s), random.nextInt(MAX_M - 3) + 3)));
            test(shuffle(chain(s, MAX_M)));

            test(shuffle(tree(almost(s), random.nextInt(MAX_M / 2) + MAX_M / 2)));
            test(shuffle(tree(s, MAX_M)));

            test(random(almost(s), 2));
            test(random(almost(s), random.nextInt(MAX_M - 3) + 3));
            test(random(s, MAX_M));

            test(minConnected(almost(s), 2));
            test(minConnected(almost(s), random.nextInt(MAX_M - 3) + 3));
            test(minConnected(s, MAX_M));

            test(maxMoves(s, random.nextInt(MAX_M - 3) + 3));
            test(maxMoves(s, MAX_M));

            test(maximize(random(almost(s), random.nextInt(MAX_M))));
            test(maximize(random(almost(s), almost(MAX_M))));
            test(minimize(random(almost(s), random.nextInt(MAX_M))));
            test(minimize(random(almost(s), almost(MAX_M))));
        }
    }

    static int almost(int n) {
        return n + random.nextInt(n / 5 + 1) - n / 5;
    }

    static int[][] antiRandom(int n, int m) {
        int[][] cave = new int[n][m];
        for (int i = 0; i < n - 1; i++) {
            Arrays.fill(cave[i], 0);
            cave[i][0] = i + 1;
        }
        return shuffle(cave);
    }

    static int[][] chain(int n, int m) {
        return extend(
                IntStream.range(0, n)
                        .mapToObj(i -> new int[]{Math.max(0, i - 1), Math.min(i + 1, n - 1)})
                        .toArray(int[][]::new),
                m
        );
    }

    static int[][] random(int n, int m) {
        int[][] cave;
        do {
            cave = Stream.generate(() -> random.ints(m, 0, n).toArray())
                    .limit(n).toArray(int[][]::new);
        } while (!isConnected(cave));
        return cave;
    }

    static int[][] extend(int[][] cave, int m) {
        return shuffle(Arrays.stream(cave).map(row ->
                IntStream.concat(
                        Arrays.stream(row),
                        random.ints(m - row.length, 0, row.length).map(i -> row[i])
                ).toArray()
        ).toArray(int[][]::new));
    }

    static int[][] maxMoves(int n, int m) {
        int[][] cave = cycle(n, m);
        cave[0][0] = n - 1;
        return cave;
    }

    static int[][] maximize(int[][] cave) {
        return optimize(cave, c -> isConnected(c) ? simulate(c) : Integer.MIN_VALUE / 2, ASC, false);
    }

    static int[][] minimize(int[][] cave) {
        return optimize(cave, c -> isConnected(c) ? simulate(c) : Integer.MAX_VALUE / 2, DESC, false);
    }

    static int[][] merge(int[][] cave1, int v1, int[][] cave2, int v2) {
        int n1 = cave1.length;
        int[][] cave = Stream.concat(
                Stream.of(cave1),
                Arrays.stream(cave2).map(a -> IntStream.of(a).map(j -> j + n1).toArray())
        ).toArray(int[][]::new);
        cave[v1][0] = v2 + n1;
        cave[v2 + n1][0] = v1;
        return cave;
    }

    private static int[][] optimize(int[][] cave, final ToIntFunction<int[][]> evaluate, final IntBinaryOperator better, boolean strict) {
        int best = evaluate.applyAsInt(cave);
        System.out.println("    Before optimization " + best);
        int improvements = 0;
        for (int i = 0; i < 10000; i++) {
            int[][] newCave = shuffle(Arrays.stream(cave).map(int[]::clone).toArray(int[][]::new));
            int f = random.nextInt(cave.length);
            int t = random.nextInt(cave.length);
            newCave[f][random.nextInt(cave[0].length)] = t;
            int moves = evaluate.applyAsInt(newCave);
            int result = better.applyAsInt(best, moves);
            if (result > 0 || result == 0 && !strict) {
                cave = newCave;
                best = moves;
                improvements++;
            }
        }
        System.out.println("    After optimization " + best + " (" + improvements + " improvements)");
        return cave;
    }

    private static int simulate(final int[][] cave) {
        return indiana_stress.simulate((m, next) -> new indiana_gk.Solution().solve(m, next), cave);
    }

    static int[][] cycle(int n, int m) {
        return extend(
                IntStream.range(0, n).mapToObj(i -> new int[]{(i + 1) % n}).toArray(int[][]::new),
                m
        );
    }
    
    static int[] shuffle(int[] row) {
        int[] result = new int[row.length];
        for (int i = 0; i < row.length; i++) {
            int j = random.nextInt(i + 1);
            result[i] = result[j];
            result[j] = row[i];
        }
        return result;
    }
    
    static int[][] shuffle(int[][] cave) {
        int[] perm = shuffle(IntStream.range(0, cave.length).toArray());
        int[][] result = new int[cave.length][];
        for (int i = 0; i < cave.length; i++) {
            result[perm[i]] = shuffle(Arrays.stream(cave[i]).map(j -> perm[j]).toArray());
        }
        return result;
    }

    static int[][] tree(int n, int m) {
        List<List<Integer>> vertices = Stream.<List<Integer>>generate(ArrayList::new)
                .limit(n).collect(Collectors.toList());
        for (int[] edge : unprufer(random.ints(n - 2, 0, n).toArray())) {
            vertices.get(edge[0]).add(edge[1]);
            vertices.get(edge[1]).add(edge[0]);
        }
        return extend(vertices.stream().map(vertex -> vertex.stream().mapToInt(a -> a).toArray()).toArray(int[][]::new), m);
    }

    static List<int[]> unprufer(int[] a) {
        int n = a.length + 2;
        List<int[]> edges = new ArrayList<>();
        int[] degree = new int[n];
        Arrays.fill(degree, 1);
        for (int i : a) {
            degree[i]++;
        }
        for (int i : a) {
            int j = 0;
            while (degree[j] != 1) j++;
            degree[j]--;
            degree[i]--;
            edges.add(new int[]{i, j});
        }
        int u = -1;
        int v = -1;
        for (int i = 0; i < n; i++) {
            if (degree[i] == 1) {
                if (u == -1) {
                    u = i;
                } else {
                    v = i;
                }
            }
        }
        edges.add(new int[]{u, v});
        return edges;
    }

    static int[][] minConnected(int n, int m) {
        int[][] cave = IntStream.range(0, n).mapToObj(i -> random.ints(m, i, i + 1).toArray()).toArray(int[][]::new);
        return optimize(cave, Tests::getConnected, ASC, true);
    }

    static void test(int[][] cave) throws FileNotFoundException {
        int n = cave.length;
        int m = cave[0].length;

        assert Arrays.stream(cave).allMatch(a -> a.length == m);
        assert Arrays.stream(cave).flatMapToInt(Arrays::stream).allMatch(p -> 0 <= p && p < n);
        assert 1 <= n && n <= MAX_N;
        assert 2 <= m && m <= MAX_M;
        assert isConnected(cave);

        try (PrintWriter out = new PrintWriter(String.format("%s/%02d", root.getName(), ++nTests))) {
            System.out.println("Writing test " + nTests + " (" + simulate(cave) + " moves)");
            out.println(e(n) + " " + e(m));
            for (int[] chamber : cave) {
                out.println(Arrays.stream(chamber)
                        .map(Tests::e)
                        .mapToObj(Integer::toString)
                        .collect(Collectors.joining(" ")));
            }
        }
    }

    static boolean isConnected(int[][] cave) {
        return getConnected(cave) == cave.length * cave.length;
    }

    private static int getConnected(final int[][] cave) {
        int n = cave.length;
        int m = cave[0].length;

        boolean[][] reachable = new boolean[n][n];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                reachable[i][cave[i][j]] = true;
            }
        }
        for (int k = 0; k < n; k++) {
            for (int i = 0; i < n; i++) {
                for (int j = 0; j < n; j++) {
                    reachable[i][j] |= reachable[i][k] && reachable[k][j];
                }
            }
        }
        int count = 0;
        for (boolean[] row : reachable) {
            for (boolean b : row) {
                if (b) {
                    count++;
                }
            }
        }
        return count;
    }
}
