import java.io.File;
import java.io.PrintWriter;
import java.util.LinkedList;
import java.util.Scanner;
import java.util.function.BiFunction;

public class indiana_gk_wa_noWhiteWin {
    static final String WHITE = "center";
    static final String GREY = "left";
    static final String BLACK = "right";
    static final String TREASURE = "treasure";

    static class V {
        int passage;
        int uplink = -1;
        int height = Integer.MAX_VALUE;
        int steps = -1;

        @Override
        public String toString() {
            return String.format("[s=%d, u=%d, h=%d, steps=%d]", passage, uplink, height, steps);
        }

        void relax(final int height, final int steps) {
            if (this.height > height) {
                this.height = height;
                this.steps = steps;
                uplink = passage;
            }
        }
    }

    static class Solution {
        public void solve(int m, BiFunction<String, Integer, String> next) {
            LinkedList<V> stack = new LinkedList<>();
            String color = WHITE;
            while (!eq(color, TREASURE)) {
                while (eq(color, WHITE)) {
                    stack.addLast(new V());
                    color = next.apply(GREY, 0);
                }
//                if (eq(color, TREASURE)) {
//                    return;
//                }

                // Find cycle
                int black = 0;
                int grey = 0;
                if (eq(color, BLACK)) {
                    while (eq(color, BLACK)) {
                        black++;
                        color = next.apply(BLACK, 0);
                    }
                } else {
                    grey = 1;
                    color = next.apply(BLACK, 0);
                }
                while (eq(color, GREY)) {
                    grey++;
                    color = next.apply(GREY, 0);
                }

                // Traverse cycle sans last element
                walk(BLACK, black, next);
                walk(GREY, grey - 1, next);

                // Update reachable heights
                if (grey > 1) {
                    int height = stack.size() - grey;
                    int steps = grey + black - 2;
                    for (V v : stack.subList(height + 1, stack.size())) {
                        v.relax(height, steps);
                    }
                }

                // Backtrack finished nodes
                while (++stack.peekLast().passage == m) {
                    V v = stack.removeLast();
                    walk(next.apply(BLACK, (v.uplink + 1) % m), v.steps, next);
                }
                color = next.apply(GREY, 1);
            }
        }

        boolean eq(final String color, final String v1) {
            return v1.equals(color);
        }

        void walk(String color, final int steps, BiFunction<String, Integer, String> next) {
            for (int i = 0; i < steps; i++) {
                color = next.apply(color, 0);
            }
        }
    }

    public static void main(String[] args) throws Exception {
        Scanner in = new Scanner(System.in);
        int m = in.nextInt();
        try (PrintWriter log = new PrintWriter(new File("indiana_gk.log"))) {
            log.println("<<< " + in.next());
            new Solution().solve(m, (side, passage) -> {
                System.out.println(passage + " " + side + " " + passage);
                System.out.flush();
                log.println(">>> " + passage + " " + side + " " + passage);
                log.flush();
                String next = in.next();
                log.println("<<< " + next);
                log.flush();
                return next;
            });
        }
    }
}
