import java.io.*
import java.util.*

/**
 * Solution for NEERC'2016 Problem I: Indiana Jones and the Uniform Cave
 * This solution requires correctness of the input.
 * @author Roman Elizarov
 */

const val DEBUG = false

fun main(args: Array<String>) {
    try {
        Scanner(System.`in`).interact()
    } catch (e: Exception) {
        PrintWriter("indiana_re.crash").use { e.printStackTrace(it) }
    }
}

enum class S { CENTER, LEFT, RIGHT, TREASURE;
    override fun toString(): String = name.toLowerCase(Locale.US)
}

fun Scanner.nextS(): S {
    nextLine()
    val str = next()
    val s = S.valueOf(str.toUpperCase(Locale.US))
    require(str == s.toString())
    return s
}

fun Scanner.interact() {
    val m = nextInt()
    require(m in 2..20)

    fun dfs(depth: Int): Int? {
        val original = nextS()
        when (original) {
            S.TREASURE -> return null
            S.CENTER -> {
                var maxLen = 0
                var maxIndex = -1
                for (i in 1..m) {
                    send("1 right 1", "at $depth -> $i")
                    val len = dfs(depth + 1) ?: return null
                    if (len > maxLen) {
                        maxLen = len
                        maxIndex = i
                    }
                }
                require(maxLen > 0)
                val j = maxIndex % m
                send("$j left $j", "at $depth back via $j")
                var leftCnt = 0
                leftCnt@ while (true) {
                    leftCnt++
                    when (nextS()) {
                        S.TREASURE, S.CENTER -> fail("Cannot happen")
                        S.LEFT -> send("0 left 0", "countLeft $leftCnt")
                        S.RIGHT -> break@leftCnt
                    }
                }
                var rightCnt = 0
                rightCnt@ while (true) {
                    rightCnt++
                    send("0 right 0", "countRight $rightCnt")
                    when (nextS()) {
                        S.TREASURE, S.CENTER -> fail("Cannot happen")
                        S.RIGHT -> continue@rightCnt
                        S.LEFT -> break@rightCnt
                    }
                }
                require(rightCnt > 0)
                for (i in 1..leftCnt) {
                    send("0 left 0", "backupLeft")
                    require(nextS() == if (i < leftCnt) S.LEFT else S.RIGHT)
                }
                repeat(rightCnt - 1) {
                    send("0 right 0", "backupRight")
                    require(nextS() == S.RIGHT)
                }
                return maxLen - 1
            }
            S.LEFT, S.RIGHT -> {
                var leftCnt = 0
                if (original == S.LEFT) {
                    leftCnt@ while (true) {
                        leftCnt++
                        send("0 left 0", "countLeft $leftCnt")
                        when (nextS()) {
                            S.TREASURE, S.CENTER -> fail("Cannot happen")
                            S.LEFT -> continue@leftCnt
                            S.RIGHT -> break@leftCnt
                        }
                    }
                }
                send("0 left 0", "markLeft")
                var rightCnt = 0
                rightCnt@ while (true) {
                    rightCnt++
                    when (nextS()) {
                        S.TREASURE, S.CENTER -> fail("Cannot happen")
                        S.RIGHT -> send("0 right 0", "countRight $rightCnt")
                        S.LEFT -> break@rightCnt
                    }
                }
                require(rightCnt > 0)
                repeat(leftCnt) {
                    send("0 left 0", "backupLeft")
                    require(nextS() == S.LEFT)
                }
                repeat(rightCnt - 1) {
                    send("0 right 0", "backupRight")
                    require(nextS() == S.RIGHT)
                }
                return rightCnt
            }
        }
    }

    check(dfs(0) == null)
}

fun send(cmd: String, debug: String) = println(cmd + if (DEBUG) " $debug" else "")

//----------------- utility ------------------

fun fail(s: String): Nothing = throw IllegalArgumentException(s)

//----------------- just for validation ------------------

/**
 * Strict scanner to verify 100% correspondence between input files and input file format specification.
 * It is a drop-in replacement for [java.util.Scanner] that could be added to a solution source
 * (cut-and-paste) without breaking its ability to work with [java.util.Scanner].
 */
class Scanner(source: InputStream) : Closeable {
    private val reader = BufferedReader(InputStreamReader(source))
    private var line: String? = ""
    private var pos: Int = 0
    private var lineNo: Int = 0

    init { nextLine() }

    override fun close() {
        require(line == null) { "Extra data at the end of file" }
        try {
            reader.close()
        } catch (e: IOException) {
            fail("Failed to close with " + e)
        }

    }

    fun nextLine() {
        require(line != null) { "EOF" }
        require(pos == line!!.length) { "Extra characters on line " + lineNo }
        try {
            line = reader.readLine()
        } catch (e: IOException) {
            fail("Failed to read line with " + e)
        }
        pos = 0
        lineNo++
    }

    operator fun next(): String {
        require(line != null) { "EOF" }
        require(line!!.isNotEmpty()) { "Empty line " + lineNo }
        if (pos == 0)
            require(line!![0] > ' ') { "Line $lineNo starts with whitespace" }
        else {
            require(pos < line!!.length) { "Line $lineNo is over" }
            require(line!![pos] == ' ') { "Wrong whitespace on line " + lineNo }
            pos++
            require(pos < line!!.length) { "Line $lineNo is over" }
            require(line!![pos] > ' ') { "Line $lineNo has double whitespace" }
        }
        val sb = StringBuilder()
        while (pos < line!!.length && line!![pos] > ' ')
            sb.append(line!![pos++])
        return sb.toString()
    }

    fun nextInt(): Int {
        val s = next()
        require(s.length == 1 || s[0] != '0') { "Extra leading zero in number $s on line $lineNo" }
        require(s[0] != '+') { "Extra leading '+' in number $s on line $lineNo" }
        try {
            return Integer.parseInt(s)
        } catch (e: NumberFormatException) {
            fail("Malformed number $s on line $lineNo")
        }
    }
}
