import java.util.Arrays;
import java.util.Random;
import java.util.function.BiFunction;

/**
 * @author Georgiy Korneev (kgeorgiy@kgeorgiy.info)
 */
public class indiana_stress {
    static int maxMoves;
    static int simulate(Solution solution, int[][] cave) {
        int n = cave.length;
        int m = cave[0].length;
        int[] stones = new int[n];
        String[] sides = new String[n];
        Arrays.fill(sides, indiana_gk.WHITE);
        boolean[][] visited = new boolean[n][m];
        int[] passages = new int[]{n * m};
        int[] chamber = new int[1];
        int[] move = new int[1];

        solution.solve(m, (side, passage) -> {
            if (++move[0] > Interact.MAX_MOVES) {
                throw new RuntimeException("Fail in " + move[0] + " moves");
            }
            assert 0 <= passage && passage < m : "passage = " + passage;

            int ch = chamber[0];
            passage = stones[ch] = (stones[ch] + passage) % m;
            sides[ch] = side;
            if (!visited[ch][passage]) {
                visited[ch][passage] = true;
                if (--passages[0] == 0) {
                    return indiana_gk.TREASURE;
                }
            }
            ch = cave[ch][passage];
            chamber[0] = ch;
//            System.out.println(ch);
            return sides[ch];

        });
        maxMoves = Math.max(maxMoves, move[0]);
        return move[0];
    }

    interface Solution {
        void solve(int m, BiFunction<String, Integer, String> next);
    }
    
    public static void stress(Solution solution) {
        maxMoves = 0;
        simulate(solution, new int[][]{{2, 1, 1}, {2, 2, 1}, {1, 1, 0}});
        simulate(solution, new int[][]{{2, 1, 2}, {1, 1, 2}, {0, 2, 2}});
        for (int n = 2; n <= Interact.MAX_N; n++) {
            System.out.println("Anti random " + n);
            simulate(solution, Tests.antiRandom(n, Math.min(n, Interact.MAX_M)));
            System.out.println("Chain " + n);
            simulate(solution, Tests.chain(n, Math.min(n, Interact.MAX_M)));
        }
        Tests.maximize(Tests.random(Interact.MAX_N, Interact.MAX_N));
        int[][] cave = null;
        try {
            Random random = new Random(4923945824503275352L);
            for (int t = 0; t < 10000; t++) {
                if (t % 1000 == 0) {
                    System.out.println("Random " + t);
                }
                int n = random.nextInt(20) + 1;
                int m = random.nextInt(20) + 1;
                if (m == 1) {
                    continue;
                }
//                int n = 20;
//                int m = 20;

                cave = Tests.random(n, m);
                simulate(solution, cave);
            }
            System.out.println("Max moves: " + maxMoves);
        } catch (Exception e) {
            System.out.println("new int[][]" + Arrays.deepToString(cave).replace("[", "{").replace("]", "}"));
            e.printStackTrace(System.out);
        }
        
    }

    public static void main(String[] args) {
        System.out.println("========== indiana_gk");
        stress((m, next) -> new indiana_gk.Solution().solve(m, next));
        System.out.println("========== indiana_gk_ok_dfs");
        stress((m, next) -> new indiana_gk_ok_dfs.Solution().solve(m, next));
        System.out.println("========== indiana_gk_wa_randomWalk");
        stress((m, next) -> new indiana_gk_wa_randomWalk.Solution().solve(m, next));
    }
}
