import javax.imageio.ImageIO;
import javax.imageio.ImageWriter;
import javax.imageio.stream.FileImageOutputStream;
import javax.imageio.stream.ImageOutputStream;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;
import java.util.List;
import java.util.function.IntConsumer;
import java.util.function.IntPredicate;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.StreamSupport;

public class Tests {
    private static final int MAX_N = 10_000;
    private static final int MAX_W = 10_000;
    private static final int MAX_H = 5_000;
    private static final int MAX_M = 5_000;
    static int nTests = 0;
    static File root = new File("tests");

    static Random random = new Random(2735038745038572342L);

    static class Move {
        int l, k;

        public Move(final int l, final int k) {
            this.l = l;
            this.k = k;
        }

        @Override
        public boolean equals(final Object o) {
            final Move move = (Move) o;
            return l == move.l && k == move.k;
        }

        @Override
        public int hashCode() {
            int result = l;
            result = 31 * result + k;
            return result;
        }
    }

    static class Level {
        boolean cross;
        long c, sumX;
        int minI, maxI, minX, maxX;
        int n;
        Set<Integer> removed = new HashSet<>();

        public Level(final boolean cross, int n) {
            this.cross = cross;
            c = n;
            minI = 0;
            maxI = n - 1;
            this.sumX = n * (long) n;
            this.n = n;
            minX = cross ? n : minI * 2 + 1;
            maxX = cross ? n : maxI * 2 + 1;
        }

        public void remove(final int k) {
            c--;
            if (cross) {
                sumX -= n;
            } else {
                sumX -= 2 * k + 1;
            }
            removed.add(k);
            while (removed.contains(minI) && minI < n - 1) minI++;
            while (removed.contains(maxI) && maxI > 0) maxI--;
            minX = cross ? n : minI * 2 + 1;
            maxX = cross ? n : maxI * 2 + 1;
        }

        public void add(final int k) {
            c++;
            if (cross) {
                sumX += n;
            } else {
                sumX += 2 * k + 1;
            }
            removed.remove(k);
            while (!removed.contains(minI) && minI > 0) minI++;
            while (!removed.contains(maxI) && maxI < n - 1) maxI--;
            minX = cross ? n : minI * 2 + 1;
            maxX = cross ? n : maxI * 2 + 1;
        }
    }

    static Move move(int l, int k) {
        return new Move(l, k);
    }

    static Move m(int l, int k) {
        return new Move(l - 1, k - 1);
    }

    static class Test {
        int n, w, h;
        List<Move> moves;

        public Test(final int n, final int w, final int h, Move... moves) {
            this(n, w, h, Arrays.asList(moves));
        }

        public Test(final int n, final int w, final int h, final List<Move> moves) {
            this.n = n;
            this.w = w;
            this.h = h;
            this.moves = new ArrayList<>(moves);
        }

        public void write() throws IOException {
            int m = moves.size();

            assert 1 <= n && n <= MAX_N;
            assert 1 <= w && w <= MAX_W;
            assert 1 <= h && h <= MAX_H;
            assert 1 <= m && m <= MAX_M;
            assert moves.stream().allMatch(mv -> 0 <= mv.l && mv.l < h && 0 <= mv.k && mv.k < n);
            assert m == new HashSet<>(moves).size();

            String fileName = String.format("%02d", ++nTests);
            try (PrintWriter out = new PrintWriter(new File(root, fileName))) {
                System.out.println("Writing test " + nTests);
                out.println(n + " " + w);
                out.println(h + " " + m);
                for (Move move : moves) {
                    out.println((move.l + 1) + " " + (move.k + 1));
                }
            }

            if (n <= 100 && h <= 100) {
                Set<Move> moveSet = new HashSet<>(moves);
                int width = x(n, 0, 0) * 2 / 10;
                int height = y(n, n, h) / 10;
                BufferedImage img = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
                Graphics2D g = (Graphics2D) img.getGraphics();
                g.setColor(Color.WHITE);
                g.fillRect(0, 0, width, height);
                g.setColor(Color.BLACK);
                g.translate(width / 2, height);
                g.scale(0.1, -0.1);
                g.setStroke(new BasicStroke(0.1f));
                g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                for (int l = 0; l < h; l++) {
                    for (int k = n - 1; k >= 0; k--) {
                        if (!moveSet.contains(move(l, k))) {
                            int x = l % 2 == 0 ? k : 0;
                            int y = k - x;
                            int dy = l % 2 == 0 ? n : 1;
                            int dx = n + 1 - dy;

                            rect(g, x, y, l, 0, dy, 0, 0, 0, 1);
                            rect(g, x, y, l, dx, 0, 0, 0, 0, 1);
                            rect(g, x, y, l + 1, 0, dy, 0, dx, 0, 0);
                        }
                    }
                }
                g.dispose();

                ImageWriter writer = ImageIO.getImageWritersByMIMEType("image/png").next();
                try (ImageOutputStream os = new FileImageOutputStream(new File(root, fileName + ".png"))) {
                    writer.setOutput(os);
                    writer.write(img);
                }
            }
        }

        private void rect(
                final Graphics g,
                final int x, final int y, final int z,
                final int dx1, final int dy1, final int dz1,
                final int dx2, final int dy2, final int dz2
        ) {
            Polygon polygon = new Polygon(
                    new int[]{x(x, y, z), x(x + dx1, y + dy1, z + dz1), x(x + dx1 + dx2, y + dy1 + dy2, z + dz1 + dz2), x(x + dx2, y + dy2, z + dz2)},
                    new int[]{y(x, y, z), y(x + dx1, y + dy1, z + dz1), y(x + dx1 + dx2, y + dy1 + dy2, z + dz1 + dz2), y(x + dx2, y + dy2, z + dz2)},
                    4
            );
            g.setColor(Color.WHITE);
            g.fillPolygon(polygon);
            g.setColor(Color.BLACK);
            g.drawPolygon(polygon);
        }

        static int SIZE = 100;
        static double A = Math.PI / 4;
        static final double sina = Math.sin(A);
        static final double cosa = Math.cos(A);
        static double B = Math.PI / 16;
        static private final double sinb = Math.sin(B);
        static private final double cosb = Math.cos(B);
        static  int x(final int x, final int y, final int z) {
            return (int) ((x * cosa - y * sina) * SIZE);
        }

        static int y(final int x, final int y, final int z) {
            return (int) ((z * cosb + (x * sina + y * cosa) * sinb) * SIZE);
        }

        static class Tower {
            private final List<Level> xLevels = new ArrayList<>();
            private final List<Level> yLevels = new ArrayList<>();
            private int h;

            public Tower(int n, int h) {
                this.h = h;
                for (int i = h - 1; i >= 0; i--) {
                    xLevels.add(new Level(i % 2 != 0, n));
                    yLevels.add(new Level(i % 2 == 0, n));
                }
            }

            public int remove(final Move move) {
                xLevels.get(h - move.l - 1).remove(move.k);
                yLevels.get(h - move.l - 1).remove(move.k);
                return Math.max(collapses(xLevels), collapses(yLevels));
            }

            public void add(final Move move) {
                xLevels.get(h - move.l - 1).add(move.k);
                yLevels.get(h - move.l - 1).add(move.k);
            }

            private int collapses(final List<Level> levels) {
                long c = 0;
                long sx = 0;
                for (int i = 0; i < levels.size(); i++) {
                    Level level = levels.get(i);
                    if (c > 0 && !level.cross && (sx <= level.minX * c || level.maxX * c <= sx)) {
                        return i;
                    }
                    c += level.c;
                    sx += level.sumX;
                }
                return -1;
            }
        }

//        public Test shuffle() {
//            Tower tower = new Tower(n, h);
//            for (int i = 0; i < moves.size(); i++) {
//                int collapse = tower.remove(moves.get(i));
//                if (collapse >= 0) {
//                    return optimize(i, collapse);
//                }
//            }
//            return this;
//        }
//
//        private Test optimize(final int mv, final int level) {
//            List<Move> mvs = new ArrayList<>(moves);
//            Collections.shuffle(mvs, random);
//            Queue<Move> queue = new ArrayDeque<>(mvs);
//
//            Tower tower = new Tower(n ,h);
//            List<Move> result = new ArrayList<>();
//            while (!queue.isEmpty()) {
//                Move move = queue.remove();
//                int collapse = tower.remove(move);
//                if (collapse != -1 && collapse != level) {
//                    tower.add(move);
//                    queue.offer(move);
//                } else {
//                    result.add(move);
//                    System.out.println(result.size());
//                }
//            }
//            System.out.println(result.size());
//            return this;
//        }
    }

    static int almost(int n) {
        int bound = (int) Math.sqrt(n);
        int i = n + random.nextInt(bound) - bound;
        assert 0 <= i && i < n;
        return i;

    }

    public static void main(String[] args) throws IOException {
        new Test(5, 2, 6, m(4, 1), m(4, 2), m(4, 5), m(5, 3), m(4, 3), m(1, 1)).write();
        new Test(3, 3, 10, m(10, 3)).write();
        new Test(2, 2, 2, m(1, 1)).write();
        new Test(4, 3, 3, m(3, 2), m(3, 3), m(1, 1), m(1, 2), m(3, 4)).write();
        randomTest(10, 10, 100, random.ints(0, 10));
        for (int n : new int[]{10, 100, 1000, MAX_N}) {
            verySkewed(n, Math.min(MAX_M, n * n));
            for (int h : new int[]{10, 100, 1000, MAX_H}) {
                int m = (int) Math.min(n * (long) h, MAX_M);
                randomTest(n, h, m, random.ints(0, h));
                randomTest(m / 3, h, m - 3, random.ints(h / 2, h / 2 + 4));
                skewedRandom(m * 2 / 5, h, almost(m), random.ints(h - 5, h));
                sides(almost(m / 2), h, almost(m - 3), gauss(h / 3, 2, 10), random.ints(h - 3, h));
            }
        }
        int h = random.nextInt(MAX_H - 5);
        skewedRandom(almost(MAX_M * 2 / 3), MAX_H, MAX_M, random.ints(h, h + 5));
        int z = random.nextInt(MAX_H - 1);
        skewedRandom(MAX_N, MAX_H, MAX_M, random.ints(z, z + 1));
    }

    private static void verySkewed(final int n, int m) throws IOException {
        List<Move> moves = new ArrayList<>();
        long sx = 0;
        long c = 0;
        int h = 0;
        while (moves.size() <= m - 2 * n) {
            double px = c == 0 ? 2 * n : sx / c;
            sx += n * 2;
            c += 2;
            for (int k = 1; k < n - 1; k++) {
                moves.add(move(h + 1, k));
            }
            sx += n * n;
            c += n;
            for (int k = 0; k < n; k++) {
                if ((sx - (2 * k + 1)) * c >= sx * (c - 1) && px > 2 * k + 2) {
                    moves.add(move(h, k));
                    c--;
                    sx -= 2 * k + 1;
                } else {
                    break;
                }
            }
            h += 2;
        }
        List<Move> ending = new ArrayList<>();
        double px = c == 0 ? 0 : sx / c;
        for (int k = n - 1; px < 2 * k + 2; k--) {
            ending.add(move(h, k));
        }
        h += 1;
        moves.addAll(0, ending);
        int fh = h;
        test(n, h, moves.stream().limit(m).map(move -> move(fh - 1 - move.l, move.k)).collect(Collectors.toList()));
    }

    private static void test(final int n, final int h, final List<Move> moves) throws IOException {
        boolean rx = random.nextBoolean();
        boolean ry = random.nextBoolean();
        List<Move> mvs = moves.stream()
                .map(move -> move.l % 2 == 0 && rx || move.l % 2 != 0 && ry ? move(move.l, n - 1 - move.k) : move)
                .collect(Collectors.toList());
        new Tests.Test(n, random.nextInt(MAX_W) + 1, h, mvs).write();
    }

    private static void randomTest(int n, int h, int m, IntStream levels) throws IOException {
        PrimitiveIterator.OfInt i = levels.iterator();
        randomTest(n, h, m, () -> new Move(i.nextInt(), random.nextInt(n)));
    }

    private static void randomTest(final int n, final int h, final int m, final Supplier<Move> moveSupplier) throws IOException {
        Set<Move> moveSet = new LinkedHashSet<>();
        while (moveSet.size() < m) {
            moveSet.add(moveSupplier.get());
        }
        List<Move> moves = new ArrayList<>(moveSet);
        Collections.shuffle(moves, random);
        test(n, h, moves);
    }

    static int next(PrimitiveIterator.OfInt i, IntPredicate p) {
        while (true) {
            int v = i.nextInt();
            if (p.test(v)) {
                return v;
            }
        }
    }

    private static void skewedRandom(int n, int h, int m, IntStream levels) throws IOException {
        int[] first = new int[h];
        PrimitiveIterator.OfInt i = levels.iterator();
        randomTest(n, h, m, () -> {
            int l = next(i, q -> first[q] < n / 2);
            return move(l, first[l]++);
        });
    }

    private static void sides(int n, int h, int m, IntStream frontLevels, IntStream backLevels) throws IOException {
        int[] front = new int[h];
        int[] back = new int[h];
        Arrays.fill(back, n);
        PrimitiveIterator.OfInt fi = frontLevels.iterator();
        PrimitiveIterator.OfInt bi = backLevels.iterator();
        randomTest(n, h, m, () -> {
            if (random.nextBoolean()){
                int l = next(fi, i -> front[i] < n / 2);
                return move(l, front[l]++);
            } else {
                int l = next(bi, i -> back[i] > n / 2);
                return move(l, --back[l]);
            }
        });
    }

    static IntStream gauss(int mean, int deviance, int samples) {
        return StreamSupport.intStream(new Spliterator.OfInt() {
            @Override
            public OfInt trySplit() {
                return this;
            }

            @Override
            public boolean tryAdvance(final IntConsumer action) {
                action.accept((int) (random.doubles(samples).sum() * 2 / samples * deviance + mean + 0.5));
                return true;
            }

            @Override
            public long estimateSize() {
                return 0;
            }

            @Override
            public int characteristics() {
                return 0;
            }
        }, false);
    }
}
