import ru.ifmo.testlib.*;

import java.io.FileNotFoundException;
import java.util.Scanner;

import static ru.ifmo.testlib.Outcome.Type.*;

public class Check implements Checker {
  private final int MAX = 10 * 1000;

  static class Picture {
    int n;
    int[] r;
    int[] c;
  }

  private Picture a;
  private Picture b;
  private Picture c;

  @Override
  public Outcome test(InStream inf, InStream ouf, InStream ans) {
    readInput(inf);
    boolean pa = readAns(ouf, WA);
    boolean ja = readAns(ans, FAIL);
    if (pa == ja) {
      return new Outcome(OK, pa ? "OK" : "Impossible");
    } else {
      if (ja) {
        return new Outcome(WA, "No solution found, but it exists");
      } else {
        return new Outcome(FAIL, "Participant found solution, but jury didn't");
      }
    }
  }

  private void readInput(InStream inf) {
    a = readPicture(inf);
    b = readPicture(inf);
    c = readPicture(inf);
  }

  private boolean readAns(InStream ouf, Outcome.Type type) {
    String ans = ouf.nextToken();
    if (ans.equals("NO")) {
      return false;
    } else if (ans.equals("YES")) {
      int dc = ouf.nextInt();
      int dr = ouf.nextInt();
      if (!(-MAX <= dc && dc <= MAX && -MAX <= dr && dr <= MAX)) {
        throw new Outcome(type, "Too far parallel translation");
      }
      if (merge(merge(a, b, dr, dc), c).n != 0) {
        throw new Outcome(type, "Picture is not same");
      }
      return true;
    } else {
      throw new Outcome(type, String.format("Expected 'YES' or 'NO', found '%s'", ans));
    }
  }

  private Picture merge(Picture a, Picture b) {
    if (a.n == 0) {
      return b;
    } else if (b.n == 0) {
      return a;
    }
    int dr = a.r[0] - b.r[0];
    int dc = a.c[0] - b.c[0];
    return merge(a, b, dr, dc);
  }

  private Picture merge(Picture a, Picture b, int dr, int dc) {
    Picture res = new Picture();
    res.r = new int[a.n + b.n];
    res.c = new int[a.n + b.n];
    int i = 0, j = 0;
    while (i < a.n || j < b.n) {
      int r1 = i < a.n ? a.r[i] : Integer.MAX_VALUE;
      int c1 = i < a.n ? a.c[i] : Integer.MAX_VALUE;
      int r2 = j < b.n ? b.r[j] + dr : Integer.MAX_VALUE;
      int c2 = j < b.n ? b.c[j] + dc : Integer.MAX_VALUE;
      if (r1 == r2 && c1 == c2) {
        i++;
        j++;
      } else if (r1 < r2 || (r1 == r2 && c1 < c2)) {
        res.r[res.n] = r1;
        res.c[res.n] = c1;
        i++;
        res.n++;
      } else {
        res.r[res.n] = r2;
        res.c[res.n] = c2;
        j++;
        res.n++;
      }
    }
    return res;
  }

  Picture readPicture(InStream in) {
    int h = in.nextInt();
    int w = in.nextInt();
    Picture picture = new Picture();
    picture.r = new int[w * h];
    picture.c = new int[w * h];
    for (int i = 0; i < h; i++) {
      String s = in.nextToken();
      for (int j = 0; j < w; j++) {
        if (s.charAt(j) == '*') {
          picture.r[picture.n] = i;
          picture.c[picture.n] = j;
          picture.n++;
        }
      }
    }
    return picture;
  }

  public static void main(String[] args) throws FileNotFoundException {
    new kids_pm().run();
  }
}
