import java.io.*;
import java.util.*;

public class Tests {
    private static File dir = new File("tests");
    private int curTest = 1;
    private static boolean verbose = true;

    private String testName;
    private PrintWriter out;
    private Random random = new Random(345340567868642L);

    public static final int MAX = 1000;
    private boolean[][] SINGLE = full(1, 1);

    public void run() {
        t(
                "..* .*. *.*",
                "**. ..* .*.",
                ".* *. ** .* *."
        );
        t(
                "** ** **",
                "*** ***",
                "..* ..* **."
        );
        t(
                "** .*",
                "*. **",
                "** **"
        );
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                if (i == j) continue;
                int k = 3 - i - j;
                String[] ss = new String[]{"***", "* * *", ".*. *.* .*."};
                t(ss[i], ss[j], ss[k]);
            }
        }
        for (int i = 0; i < 3; i++) {
            for (int j = -2; j < 3; j++) {
                if (i == 0 && j < 0) continue;
                t(SINGLE, SINGLE, merge(SINGLE, SINGLE, i, j), "Two single freckles, shift = (" + i + ", " + j + ")");
            }
        }

        for (int i = 0; i < 12; i++) {
            random2(20, i % 3, false);
        }
        random2(20, 0, true);

        int N = MAX;
        for (int l1 = 0; l1 < 4; l1++) {
            for (int r1 = l1 + 1; r1 < 4; r1++) {
                int l2 = 0;
                while (l2 == l1 || l2 == r1) l2++;
                int r2 = l2 + 1;
                while (r2 == l1 || r2 == r1) r2++;
                char[] cc = new char[4];
                cc[l1] = '(';
                cc[r1] = ')';
                cc[l2] = '[';
                cc[r2] = ']';
                for (int t1 = 0; t1 < 4; t1++) {
                    for (int b1 = t1 + 1; b1 < 4; b1++) {
                        int t2 = 0;
                        while (t2 == t1 || t2 == b1) t2++;
                        int b2 = t2 + 1;
                        while (b2 == t1 || b2 == b1) b2++;
                        char[] cc2 = new char[4];
                        cc2[t1] = '(';
                        cc2[b1] = ')';
                        cc2[t2] = '[';
                        cc2[b2] = ']';

                        for (int k = 0; k < 3; k++) {

                            if (Math.max(l1, l2) < Math.min(r1, r2) && Math.max(t1, t2) < Math.min(b1, b2)) {
                                int[] x = new int[4];
                                for (int i = 0; i < 4; i++) x[i] = N / 4 * i + random.nextInt(N / 4);
                                int[] y = new int[4];
                                for (int i = 0; i < 4; i++) y[i] = N / 4 * i + random.nextInt(N / 4);
                                boolean[][] a = randomPicture2(y[b1] - y[t1], x[r1] - x[l1]);
                                boolean[][] b = randomPicture2(y[b2] - y[t2], x[r2] - x[l2]);
                                boolean[][] c = merge(a, b, y[t2] - y[t1], x[l2] - x[l1]);

                                if (k == 1) {
                                    boolean[][] t = a; a = c; c = t;
                                }
                                if (k == 2) {
                                    boolean[][] t = b; b = c; c = t;
                                }

                                t(a, b, c, "Two spots: " + new String(cc) + ", " + new String(cc2));
                            }
                        }
                    }
                }

            }
        }

        for (int i = 0; i < 3; i++) {
            random2(MAX * 8 / 10, i, false);
        }
        random2(MAX * 8 / 10, 0, true);
        {
            boolean[][] a = chess(MAX / 2, MAX / 2, false);
            boolean[][] b = chess(MAX / 2, MAX / 2, false);
            boolean[][] c = randomMerge(a, b);
            t(a, b, c, "Two chessboards");
        }
        {
            boolean[][] a = full(MAX / 2, MAX / 2);
            boolean[][] b = full(MAX / 2, MAX / 2);
            boolean[][] c = randomMerge(a, b);
            t(a, b, c, "Two rectangles");
        }
        {
            t(full(MAX, MAX), full(MAX, MAX), full(MAX, MAX), "Three max full fields");
            t(full(MAX, MAX), full(MAX, MAX), empty(MAX, MAX), "Two max full fields and empty");
            t(onerandom(MAX, MAX), onerandom(MAX, MAX), empty(MAX, MAX), "Two freckles on a big field");
            t(chess(MAX, MAX, true), chess(MAX, MAX, false), full(MAX, MAX), "Two max chessboards to full");
            t(chess(MAX, MAX, true), chess(MAX, MAX, true), full(MAX, MAX), "Two same max chessboards to full");
            t(chess(MAX, MAX, true), chess(MAX, MAX, true), empty(MAX, MAX), "Two same max chessboards to empty");
            t(full(1, MAX), full(MAX, 1), randomMerge(full(1, MAX), full(MAX, 1)), "Two lines");
            t(SINGLE, SINGLE, merge(SINGLE, SINGLE, MAX - 1, MAX - 1), "Two freckles in opposite corners");
        }
    }

    private boolean[][] onerandom(int h, int w) {
        boolean[][] res = empty(h, w);
        res[random.nextInt(h)][random.nextInt(w)] = true;
        return res;
    }

    private boolean[][] full(int h, int w) {
        boolean[][] res = new boolean[h][w];
        for (boolean[] re : res) {
            Arrays.fill(re, true);
        }
        return res;
    }

    private boolean[][] empty(int h, int w) {
        return new boolean[h][w];
    }

    private boolean[][] chess(int h, int w, boolean start) {
        boolean[][] res = new boolean[h][w];
        for (int i = 0; i < h; i++) {
            for (int j = 0; j < w; j++) {
                res[i][j] = ((i + j) % 2 == 0) ^ start;
            }
        }
        return res;
    }

    private void t(String a, String b, String c) {
        t(parse(a), parse(b), parse(c), a + ", " + b + ", " + c);
    }

    private boolean[][] parse(String c) {
        String[] ss = c.split(" ");
        int h = ss.length;
        int w = ss[0].length();
        boolean[][] res = new boolean[h][w];
        for (int i = 0; i < h; i++) {
            for (int j = 0; j < w; j++) {
                res[i][j] = ss[i].charAt(j) == '*';
            }
        }
        return res;
    }

    private void random2(int n, int q, boolean impossible) {
        random2(n, n, n, n, q, impossible);
    }

    private void random2(int h1, int w1, int h2, int w2, int q, boolean impossible) {
        boolean[][] a = randomPicture2(h1, w1);
        boolean[][] b = randomPicture2(h2, w2);
        boolean[][] c = randomMerge(a, b);
        if (impossible) {
            for (int i = 0; i < 2; i++) {
                int x = random.nextInt(c.length);
                int y = random.nextInt(c[0].length);
                c[x][y] ^= true;
            }
        }
        String size = a.length + "x" + a[0].length + ", " + b[0].length + "x" + b[0].length;
        if (q == 0) t(a, b, c, "Random spots " + size + ", merge is third" + (impossible ? ", impossible" : ""));
        if (q == 1) t(a, c, b, "Random spots " + size + ", merge is second" + (impossible ? ", impossible" : ""));
        if (q == 2) t(c, b, a, "Random spots " + size + ", merge is first" + (impossible ? ", impossible" : ""));
    }

    private void random(int n, int q) {
        boolean[][] a = randomPicture(n);
        boolean[][] b = randomPicture(n);
        boolean[][] c = randomMerge(a, b);
        if (q == 0) t(a, b, c, "Random fields, merge is third");
        if (q == 1) t(a, c, b, "Random fields, merge is second");
        if (q == 2) t(c, b, a, "Random fields, merge is first");
    }

    private boolean[][] randomMerge(boolean[][] a, boolean[][] b) {
        int h1 = a.length;
        int w1 = a[0].length;
        int h2 = b.length;
        int w2 = b[0].length;
        int dr2 = -h2 / 2 + random.nextInt(h1);
        int dc2 = -w2 / 2 + random.nextInt(w1);
        return merge(a, b, dr2, dc2);
    }

    private boolean[][] merge(boolean[][] a, boolean[][] b, int dr2, int dc2) {
        int h1 = a.length;
        int w1 = a[0].length;
        int h2 = b.length;
        int w2 = b[0].length;
        int dr1 = 0;
        int dc1 = 0;
        if (dc2 < 0) {
            dc1 = -dc2;
            dc2 = 0;
        }
        if (dr2 < 0) {
            dr1 = -dr2;
            dr2 = 0;
        }
        int h = Math.max(h1 + dr1, h2 + dr2);
        int w = Math.max(w1 + dc1, w2 + dc2);

        if (h > MAX) {
            h = MAX;
            dr1 = Math.min(dr1, h - h1);
            dr2 = Math.min(dr2, h - h2);
        }
        if (w > MAX) {
            w = MAX;
            dc1 = Math.min(dc1, w - w1);
            dc2 = Math.min(dc2, w - w2);
        }

        boolean[][] res = new boolean[h][w];
        for (int i = 0; i < h1; i++) {
            for (int j = 0; j < w1; j++) {
                if (a[i][j]) res[i + dr1][j + dc1] ^= true;
            }
        }
        for (int i = 0; i < h2; i++) {
            for (int j = 0; j < w2; j++) {
                if (b[i][j]) res[i + dr2][j + dc2] ^= true;
            }
        }
        return res;
    }

    private boolean[][] randomPicture(int h, int w) {
        w -= random.nextInt(w / 2);
        h -= random.nextInt(h / 2);
        boolean[][] res = new boolean[h][w];
        for (int i = 0; i < h; i++) {
            for (int j = 0; j < w; j++) {
                res[i][j] = random.nextBoolean();
            }
        }
        return res;
    }

    private boolean[][] randomPicture(int n) {
        return randomPicture(
                n - random.nextInt(n / 2),
                n - random.nextInt(n / 2)
        );
    }

    private boolean[][] randomPicture2(int h, int w) {
//        w -= random.nextInt(w / 2);
//        h -= random.nextInt(h / 2);
//        System.out.println(h + " " + w);
        boolean[][] res = new boolean[h][w];
        for (int i = 0; i < h; i++) {
            for (int j = 0; j < w; j++) {
                double d = dd(2.0 * (i + 1) / (h + 1) - 1, 2.0 * (j + 1) / (w + 1) - 1);
                res[i][j] = random.nextDouble() < (1 - d);
            }
        }
        return res;
    }

    private boolean[][] randomPicture2(int n) {
        return randomPicture2(n, n);
    }

    private double dd(double dx, double dy) {
        return dx * dx + dy * dy;
    }

    private void t(boolean[][] a, boolean[][] b, boolean[][] c, String comment) {
        open(comment);
        check(a);
        check(b);
        print(a);
        print(b);
        print(c);
        close();
    }

    private void check(boolean[][] a) {
        for (boolean[] booleen : a) {
            for (boolean b : booleen) {
                if (b) return;
            }
        }
        throw new RuntimeException();
    }

    private void print(boolean[][] pic) {
        out.println(pic.length + " " + pic[0].length);
        for (boolean[] line : pic) {
            for (boolean b : line) {
                out.print(b ? "*" : ".");
            }
            out.println();
        }
    }

    private void open(String comment) {
        try {
            testName = "" + curTest / 10 + curTest % 10;
            out = new PrintWriter(new File(dir, testName));
            if (verbose) {
                System.out.print("Generating " + testName + ": " + comment + "... ");
            }
        } catch (IOException e) {
            throw new RuntimeException("Unable to open " + testName + " for writing", e);
        }
    }

    private void close() {
        out.close();
        if (verbose) {
            System.out.println(" done.");
        }
        curTest++;
    }

    public static void main(String[] args) {
        Locale.setDefault(Locale.US);
        dir.mkdirs();
        new Tests().run();
    }
}
