@file:Suppress("PackageDirectoryMismatch")

package kids_re_wa

import java.io.*
import kotlin.system.measureTimeMillis
import kids_re_wa.Output.*
/**
 * Solution for NEERC'2016 Problem K: Kids Designing Kids
 * This solution requires correctness of the input.
 * @author Roman Elizarov
 */

const val INPUT = "kids.in"
const val OUTPUT = "kids.out"

const val MAX_HW = 1000

fun main(args: Array<String>) {
    val time = measureTimeMillis {
        val input = Scanner(File(INPUT)).use { it.readInput() }
        val output = input.solve()
        PrintWriter(OUTPUT).use { it.writeOutput(output) }
    }
    println("Done in $time ms")
}

// input

class Pic(val h: Int, val w: Int) {
    val a = Array(w) { BooleanArray(h) }

    operator fun get(x: Int, y: Int): Boolean =
            if (x in 0..w - 1 && y in 0..h - 1) a[x][y] else false

    operator fun set(x: Int, y: Int, v: Boolean) {
        a[x][y] = v
    }

    fun isEmpty(): Boolean {
        for (x in 0..w - 1)
            for (y in 0..h - 1)
                if (get(x, y)) return false
        return true
    }
}

class Input(val kevin: Pic, val kimberly: Pic, val child: Pic)

fun Scanner.readPic(): Pic = Pic(nextInt(), nextInt()).apply {
    nextLine()
    check(h in 1..MAX_HW)
    check(w in 1..MAX_HW)
    for (i in 0..h - 1) {
        val line = next()
        nextLine()
        check(line.length == w)
        for (j in 0..w - 1)
            when (line[j]) {
                '.' -> a[j][i] = false
                '*' -> a[j][i] = true
                else -> fail("Wrong char")
            }
    }
}

fun Scanner.readInput() : Input = Input(readPic(), readPic(), readPic()).apply {
    check(!kevin.isEmpty())
    check(!kimberly.isEmpty())
}

// solution

sealed class Output {
    object NotFound : Output()

    class Pt(val x: Int, val y: Int) : Output() {
        operator fun minus(p: Pt): Pt = Pt(x - p.x, y - p.y)
        operator fun plus(p: Pt): Pt = Pt(x + p.x, y + p.y)
        operator fun unaryMinus(): Pt = Pt(-x, -y)
    }
}

fun Pic.find(): Pt {
    for (x in 0..w - 1)
        for (y in 0..h - 1)
            if (a[x][y]) return Pt(x, y)
    return Pt(0, 0)
}

fun max(x: Int, y: Int) = Math.max(x, y)

fun mix(p: Pic, q: Pic, d: Pt): Pic {
    val px = max(0, -d.x)
    val py = max(0, -d.y)
    val qx = max(0, d.x)
    val qy = max(0, d.y)
    val w = max(p.w + px, q.w + qx)
    val h = max(p.h + py, q.h + qy)
    val r = Pic(h, w)
    for (x in 0..w - 1)
        for (y in 0..h - 1)
            r[x, y] = p[x - px, y - py] xor q[x - qx, y - qy]
    return r
}

enum class Ret { DPQ, DPR }

fun check(p: Pic, q: Pic, r: Pic, ret: Ret): Pt?  {
    val p0 = p.find()
    val q0 = q.find()
    val dpq = p0 - q0
    val pq = mix(p, q, dpq)
    if (pq.isEmpty()) {
        if (r.isEmpty()) return when (ret) {
            Ret.DPQ -> dpq
            Ret.DPR -> null
        } else
            return null
    }
    val pq0 = pq.find()
    val r0 = r.find()
    val dpqr = pq0 - r0
    val pqr = mix(pq, r, dpqr)
    return if (pqr.isEmpty()) when (ret) {
        Ret.DPQ -> dpq
        Ret.DPR -> dpq + dpqr
    } else null
}

fun Input.solve(): Output {
    check(kevin, kimberly, child, Ret.DPQ)?.let { return it }
    check(kevin, child, kimberly, Ret.DPR)?.let { return it }
    check(kimberly, child, kevin, Ret.DPR)?.let { return -it }
    return NotFound
}

// result

fun PrintWriter.writeOutput(output: Output) {
    when (output) {
        is NotFound -> println("NO")
        is Pt -> {
            println("YES")
            println("${output.x} ${output.y}")
        }
    }
}

//----------------- utility ------------------

fun fail(s: String): Nothing = throw IllegalArgumentException(s)

//----------------- just for validation ------------------

/**
 * Strict scanner to verify 100% correspondence between input files and input file format specification.
 * It is a drop-in replacement for [java.util.Scanner] that could be added to a solution source
 * (cut-and-paste) without breaking its ability to work with [java.util.Scanner].
 */
class Scanner(source: File) : Closeable {
    private val reader = BufferedReader(FileReader(source))
    private var line: String? = ""
    private var pos: Int = 0
    private var lineNo: Int = 0

    init { nextLine() }

    override fun close() {
        require(line == null) { "Extra data at the end of file" }
        try {
            reader.close()
        } catch (e: IOException) {
            fail("Failed to close with " + e)
        }

    }

    fun nextLine() {
        require(line != null) { "EOF" }
        require(pos == line!!.length) { "Extra characters on line " + lineNo }
        try {
            line = reader.readLine()
        } catch (e: IOException) {
            fail("Failed to read line with " + e)
        }
        pos = 0
        lineNo++
    }

    operator fun next(): String {
        require(line != null) { "EOF" }
        require(line!!.isNotEmpty()) { "Empty line " + lineNo }
        if (pos == 0)
            require(line!![0] > ' ') { "Line $lineNo starts with whitespace" }
        else {
            require(pos < line!!.length) { "Line $lineNo is over" }
            require(line!![pos] == ' ') { "Wrong whitespace on line " + lineNo }
            pos++
            require(pos < line!!.length) { "Line $lineNo is over" }
            require(line!![pos] > ' ') { "Line $lineNo has double whitespace" }
        }
        val sb = StringBuilder()
        while (pos < line!!.length && line!![pos] > ' ')
            sb.append(line!![pos++])
        return sb.toString()
    }

    fun nextInt(): Int {
        val s = next()
        require(s.length == 1 || s[0] != '0') { "Extra leading zero in number $s on line $lineNo" }
        require(s[0] != '+') { "Extra leading '+' in number $s on line $lineNo" }
        try {
            return Integer.parseInt(s)
        } catch (e: NumberFormatException) {
            fail("Malformed number $s on line $lineNo")
        }

    }
}
