import java.io.*;
import java.util.*;

public class Tests {
    private static File dir = new File("tests");
    private int curTest = 1;
    private static boolean verbose = true;

    private String testName;
    private PrintWriter out;
    private Random r = new Random(345340567868642L);
    
    public static final long MAX = 1_000_000_000_000_000_000L;
    public static final int MAX_LENGTH = 100_000;


    public void run() {
        t(1, 35);
        t(36, 41);
        t(8, 8);
        t(36, 36);
        t(37, 50);
        t(239, 566);
        for (int j = 1; j <= 5; j++) {
            for (int i = 1; i <= j; i++) {
                t(i, j);
                t(i * 10, j * 10);
                t(i * 100, j * 100);
                t(i * 1000, j * 1000);
            }
        }
        for (int i = 0; i < 3; i++) {
            t(1 + MAX_LENGTH * i, MAX_LENGTH * (i + 1));
            t(MAX - MAX_LENGTH * (i + 1), MAX - MAX_LENGTH * i);
            t(MAX - i, MAX - i);
            long x;
            do {
                x = r.nextLong();
            } while (x < 1 || x + MAX_LENGTH > MAX);
            t(x, x + MAX_LENGTH);
        }
        {
        	long x = 943177045245549867L; // index of last singleton
        	for (int i = -1; i < 2; i++) {
        		t(x + i, x + 5 - i);
        	}
        	t(x + 6, x + 7);
        	t(x + 4, x + 4 + MAX_LENGTH);
        	t(x - MAX_LENGTH, x);
        }
    }

    private void t(long a, long b) {
        open();
        out.println(a + " " + b);
        close();
    }

    private void open() {
        try {
            testName = "" + curTest / 10 + curTest % 10;
            out = new PrintWriter(new File(dir, testName));
            if (verbose) { 
                System.out.print("Generating " + testName + "...");
            }
        } catch (IOException e) {
            throw new RuntimeException("Unable to open " + testName + " for writing", e);
        }
    }

    private void close() {
        out.close();
        if (verbose) {
            System.out.println(" done.");
        }
        curTest++;
    }

    public static void main(String[] args) {
        Locale.setDefault(Locale.US);
        dir.mkdirs();
        new Tests().run();
    }
}
