import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Random;
import java.util.Locale;

/**
 * Created by Borys Minaiev on 20.11.16.
 */
public class Tests {

	static Random rnd = new Random(12345);
	static File testsDirectory;
	static int currentTest;
	static final int MAX = (int) 1e5;

	static void printTest(TestCase t) {
		currentTest++;
		try {
			PrintWriter out = new PrintWriter(new File(testsDirectory,
					String.format(Locale.US, "%02d", currentTest)));
			out.println(t.food.length + " " + t.pos.length);
			for (int i = 0; i < t.food.length; i++) {
				out.print(Math.min(t.food[i], t.pos.length)
						+ (i == t.food.length - 1 ? String.format("%n") : " "));
			}
			for (int i = 0; i < t.pos.length; i++) {
				out.print((1 + t.pos[i]) + (i == t.pos.length - 1 ? String.format("%n") : " "));
			}
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	static class TestCase {
		int[] pos;
		int[] food;

		public TestCase(int[] pos, int[] food) {
			super();
			this.pos = pos;
			this.food = food;
		}

	}

	static void genRandomTestCase(int treeSize, int moles) {
		int[] pos = new int[moles];
		for (int i = 0; i < moles; i++) {
			pos[i] = rnd.nextInt(treeSize);
		}
		int[] food = new int[treeSize];
		int totalFood = moles;
		double[] coef = new double[] { 0.01, 0.1, 0.5, 1.0, 2.0 };
		int add = (int) (rnd.nextInt(moles) * coef[rnd.nextInt(coef.length)]);
		totalFood += add;
		for (int i = 0; i < totalFood; i++) {
			food[rnd.nextInt(food.length)]++;
		}
		printTest(new TestCase(pos, food));
	}

	static void genRandomTestCaseWithPositions(int treeSize, int moles,
			int foodCnt, int molesCnt, boolean exact) {
		int[] foodPos = new int[foodCnt];
		for (int i = 0; i < foodCnt; i++) {
			foodPos[i] = rnd.nextInt(treeSize);
		}
		int[] molesPos = new int[molesCnt];
		for (int i = 0; i < molesCnt; i++) {
			molesPos[i] = rnd.nextInt(treeSize);
		}
		int[] pos = new int[moles];
		for (int i = 0; i < moles; i++) {
			pos[i] = molesPos[rnd.nextInt(molesPos.length)];
		}
		int[] food = new int[treeSize];
		int totalFood = moles;
		double[] coef = new double[] { 0.01, 0.1, 0.5, 1.0, 2.0 };
		int add = (int) (rnd.nextInt(moles) * coef[rnd.nextInt(coef.length)]);
		if (exact) {
			add = 0;
		}
		totalFood += add;
		for (int i = 0; i < totalFood; i++) {
			food[foodPos[rnd.nextInt(foodPos.length)]]++;
		}
		printTest(new TestCase(pos, food));
	}

	static void generateSameples() {
		printTest(new TestCase(new int[] { 1, 3, 4, 1 }, new int[] { 0, 0, 5,
				1, 1 }));
	}

	static void generateTests() {
		for (int M : new int[] { 100, MAX }) {
			for (int i = 0; i < 15; i++) {
				genRandomTestCase(1 + rnd.nextInt(M), 1 + rnd.nextInt(M));
			}
			for (int i = 0; i < 15; i++) {
				int treeSize = 1 + rnd.nextInt(M);
				int moles = 1 + rnd.nextInt(M);
				int foodCnt = 1 + rnd.nextInt(treeSize);
				int molesCnt = 1 + rnd.nextInt(treeSize);
				genRandomTestCaseWithPositions(treeSize, moles, foodCnt,
						molesCnt, false);
			}
		}
		for (int cntFood : new int[] { 1, 2, 5, 33 }) {
			genRandomTestCaseWithPositions(MAX, MAX, cntFood, MAX, true);
			genRandomTestCaseWithPositions(MAX, MAX, cntFood,
					1 + rnd.nextInt(MAX), true);
		}
		for (int cntMoles : new int[] { 1, 2, 5, 33 }) {
			genRandomTestCaseWithPositions(MAX, MAX, 1, cntMoles, true);
			genRandomTestCaseWithPositions(MAX, MAX, 1 + rnd.nextInt(MAX),
					cntMoles, true);
		}
		for (int i = 0; i < 3; i++) {
			genRandomTestCaseWithPositions(MAX, MAX, 1, 1, true);
		}
	}

	public static void main(String[] args) {
		testsDirectory = new File("tests");
		if (!testsDirectory.exists()) {
			if (!testsDirectory.mkdirs()) {
				System.err.println("couldn't create tests directory");
				System.exit(1);
			}
		}
		currentTest = 0;
		generateSameples();
		generateTests();
	}
}
