import java.io.*;

public class CheckScanner {
    public static final String DIGITS = "0123456789";
    public static final String INTEGER = "+-" + DIGITS;
    public static final String ENGLISH_LOWER = "abcdefghijklmnopqrstuvwxyz";
    public static final String ENGLISH_UPPER = ENGLISH_LOWER.toUpperCase();
    public static final String ENGLISH_ALL   = ENGLISH_UPPER + ENGLISH_LOWER;

    private final InputStream is;
    private char ch;
    private int line;
    private int column;

    public CheckScanner(File file) {
        try {
            is = new FileInputStream(file);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        nextChar();
    }

    private void nextChar() {
        int b;
        try {
            b = is.read();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        if (b == -1) {
            throw new RuntimeException("Unexpected end of file");
        }
        if (b < ' ' && b != 10 && b != 13 || b > 127) {
            throw new RuntimeException("Invalid character #" + b);
        }
        if (b == 10) {
            line++;
            column = 1;
        } else {
            column++;
        }
        ch = (char) b;
    }

    public String next(String chars, int length) {
        return next(chars, length, length);
    }

    public String next(String chars, int minLength, int maxLength) {
        if (ch == 10) {
            throw new RuntimeException(location() + "Unexpected EOLN");
        }
        if (ch == ' ') {
            nextChar();
        }
        StringBuilder sb = new StringBuilder();
        while (chars.indexOf(ch) != -1) {
            sb.append(ch);
            nextChar();
        }
        if (sb.length() < minLength) {
            throw new RuntimeException("String too short: " + sb);
        }
        if (sb.length() > maxLength) {
            throw new RuntimeException("String too long: " + sb);
        }
        if (ch == 10) {
            throw new RuntimeException("LF without CR");
        }
        if (ch == 13) {
            nextChar();
            if (ch != 10) {
                throw new RuntimeException("CR without Lf");
            }
        }
        if (ch != ' ' && ch != 10) {
            throw new RuntimeException("Whitespace expected");
        }
        return sb.toString();
    }

    public int nextInt(int min, int max) {
        String string = next(INTEGER, 1, 100);
        int value = Integer.parseInt(string);
        if (value < min || value > max) {
            throw new RuntimeException("Value out of range " + value);
        }
        return value;
    }

    public long nextLong(long min, long max) {
        String string = next(INTEGER, 1, 100);
        long value = Long.parseLong(string);
        if (value < min || value > max) {
            throw new RuntimeException("Value out of range " + value);
        }
        return value;
    }

    public void eoln() {
        if (ch != 10) {
            throw new RuntimeException("EOLN expected");
        }
        ch = ' ';
    }

    private String location() {
        return line + ":" + column + ": ";
    }

    public void close() {
        try {
//          eoln();
            if (is.read() != -1) {
                throw new RuntimeException("EOF expected");
            }
            is.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public String nextLine(int minLength, int maxLength) {
        if (ch == 10) {
            throw new RuntimeException(location() + "Unexpected EOLN");
        }
        if (ch == ' ') {
            nextChar();
        }
        StringBuilder sb = new StringBuilder();
        while (ch >= ' ') {
            sb.append(ch);
            nextChar();
        }
        if (sb.length() < minLength) {
            throw new RuntimeException("Line too short: " + sb);
        }
        if (sb.length() > maxLength) {
            throw new RuntimeException("Line too long: " + sb);
        }
        if (ch == 10) {
            throw new RuntimeException("LF without CR");
        }
        if (ch == 13) {
            nextChar();
            if (ch != 10) {
                throw new RuntimeException("CR without Lf");
            }
        }
        if (ch != ' ' && ch != 10) {
            throw new RuntimeException("Whitespace expected");
        }
        return sb.toString();
    }
}
