import java.io.*;

class CheckScanner2 {
    final BufferedReader reader;
    int row, col;
    int c;

    public CheckScanner2(File file) throws FileNotFoundException {
        reader = new BufferedReader(new FileReader(file));
        row = 1;
        col = 0;
        c = ' ';
    }

    public void nextChar() {
        check(c != -1, "Read past end of file");
        try {
            c = reader.read();
            if (c == '\r') {
                c = reader.read();
                check(c == '\n', "CR not followed by LF");
            }
            col++;
        } catch (IOException e) {
            throw new AssertionError(e);
        }
    }

    public String next() {
        check(c == ' ', "Single space expected");
        nextChar();
        final StringBuilder sb = new StringBuilder();
        while (c != -1 && c != ' ' && c != '\n') {
            sb.append((char) c);
            nextChar();
        }
        check(sb.length() > 0, "Extra space");
        return sb.toString();
    }

    public int nextInt(int min, int max) {
        int value = nextInt();
        check(min <= value && value <= max, "Integer %d out of range %d..%d", value, min, max);
        return value;
    }

    private int nextInt() {
        final String token = next();
        check(token.charAt(0) != '+', "Integer number with + sign: %s", token);
        try {
            return Integer.parseInt(token);
        } catch (NumberFormatException e) {
            throw assertionError("Invalid integer number: %s", e.getMessage());
        }
    }

    private void check(boolean assertion, String format, Object... args) {
        if (!assertion) {
            throw assertionError(format, args);
        }
    }

    private AssertionError assertionError(String format, Object ... args) {
        return new AssertionError(row + ":" + col + ": " + String.format(format, args));
    }

    public void close() {
        checkEOF();
    }

    public void checkEOLN() {
        check(c == '\n', "End of line expected, but '%c' (%d) found", c, c);
        row++;
        col = 0;
        c = ' ';
    }

    public void checkEOF() {
        nextChar();
        check(c == -1, "End of file expected");
    }
}
